################################################################################
#  THIS FILE IS 100% GENERATED BY ZPROJECT; DO NOT EDIT EXCEPT EXPERIMENTALLY  #
#  Read the zproject/README.md for information about making permanent changes. #
################################################################################
from . import utils
from . import destructors
libczmq_destructors = destructors.lib

class Zpoller(object):
    """
    event-driven reactor
    """

    def __init__(self, reader, *reader_args):
        """
        Create new poller, specifying zero or more readers. The list of
        readers ends in a NULL. Each reader can be a zsock_t instance, a
        zactor_t instance, a libzmq socket (void *), or a file handle.
        """
        p = utils.lib.zpoller_new(reader._p, *reader_args)
        if p == utils.ffi.NULL:
            raise MemoryError("Could not allocate person")

        # ffi.gc returns a copy of the cdata object which will have the
        # destructor called when the Python object is GC'd:
        # https://cffi.readthedocs.org/en/latest/using.html#ffi-interface
        self._p = utils.ffi.gc(p, libczmq_destructors.zpoller_destroy_py)

    def add(self, reader):
        """
        Add a reader to be polled. Returns 0 if OK, -1 on failure. The reader may
        be a libzmq void * socket, a zsock_t instance, a zactor_t instance or a
        file handle.
        """
        return utils.lib.zpoller_add(self._p, reader._p)

    def remove(self, reader):
        """
        Remove a reader from the poller; returns 0 if OK, -1 on failure. The reader
        must have been passed during construction, or in an zpoller_add () call.
        """
        return utils.lib.zpoller_remove(self._p, reader._p)

    def set_nonstop(self, nonstop):
        """
        By default the poller stops if the process receives a SIGINT or SIGTERM
        signal. This makes it impossible to shut-down message based architectures
        like zactors. This method lets you switch off break handling. The default
        nonstop setting is off (false).
        """
        utils.lib.zpoller_set_nonstop(self._p, nonstop)

    def wait(self, timeout):
        """
        Poll the registered readers for I/O, return first reader that has input.
        The reader will be a libzmq void * socket, a zsock_t, a zactor_t
        instance or a file handle as specified in zpoller_new/zpoller_add. The
        timeout should be zero or greater, or -1 to wait indefinitely. Socket
        priority is defined by their order in the poll list. If you need a
        balanced poll, use the low level zmq_poll method directly. If the poll
        call was interrupted (SIGINT), or the ZMQ context was destroyed, or the
        timeout expired, returns NULL. You can test the actual exit condition by
        calling zpoller_expired () and zpoller_terminated (). The timeout is in
        msec.
        """
        return utils.lib.zpoller_wait(self._p, timeout)

    def expired(self):
        """
        Return true if the last zpoller_wait () call ended because the timeout
        expired, without any error.
        """
        return utils.lib.zpoller_expired(self._p)

    def terminated(self):
        """
        Return true if the last zpoller_wait () call ended because the process
        was interrupted, or the parent context was destroyed.
        """
        return utils.lib.zpoller_terminated(self._p)

    @staticmethod
    def test(verbose):
        """
        Self test of this class.
        """
        utils.lib.zpoller_test(verbose)

################################################################################
#  THIS FILE IS 100% GENERATED BY ZPROJECT; DO NOT EDIT EXCEPT EXPERIMENTALLY  #
#  Read the zproject/README.md for information about making permanent changes. #
################################################################################



################################################################################
#  THIS FILE IS 100% GENERATED BY ZPROJECT; DO NOT EDIT EXCEPT EXPERIMENTALLY  #
#  Read the zproject/README.md for information about making permanent changes. #
################################################################################
from . import utils
from . import destructors
libczmq_destructors = destructors.lib

class Zcertstore(object):
    """
    work with CURVE security certificate stores
    """

    def __init__(self, location):
        """
        Create a new certificate store from a disk directory, loading and
        indexing all certificates in that location. The directory itself may be
        absent, and created later, or modified at any time. The certificate store
        is automatically refreshed on any zcertstore_lookup() call. If the
        location is specified as NULL, creates a pure-memory store, which you
        can work with by inserting certificates at runtime.
        """
        p = utils.lib.zcertstore_new(utils.to_bytes(location))
        if p == utils.ffi.NULL:
            raise MemoryError("Could not allocate person")

        # ffi.gc returns a copy of the cdata object which will have the
        # destructor called when the Python object is GC'd:
        # https://cffi.readthedocs.org/en/latest/using.html#ffi-interface
        self._p = utils.ffi.gc(p, libczmq_destructors.zcertstore_destroy_py)

    def set_loader(self, loader, destructor, state):
        """
        Override the default disk loader with a custom loader fn.
        """
        utils.lib.zcertstore_set_loader(self._p, loader, destructor, state)

    def lookup(self, public_key):
        """
        Look up certificate by public key, returns zcert_t object if found,
        else returns NULL. The public key is provided in Z85 text format.
        """
        return utils.lib.zcertstore_lookup(self._p, utils.to_bytes(public_key))

    def insert(self, cert_p):
        """
        Insert certificate into certificate store in memory. Note that this
        does not save the certificate to disk. To do that, use zcert_save()
        directly on the certificate. Takes ownership of zcert_t object.
        """
        utils.lib.zcertstore_insert(self._p, utils.ffi.new("zcert_t **", cert_p._p))

    def empty(self):
        """
        Empty certificate hashtable. This wrapper exists to be friendly to bindings,
        which don't usually have access to struct internals.
        """
        utils.lib.zcertstore_empty(self._p)

    def print_py(self):
        """
        Print list of certificates in store to logging facility
        """
        utils.lib.zcertstore_print(self._p)

    def certs(self):
        """
        Return a list of all the certificates in the store.
        The caller takes ownership of the zlistx_t object and is responsible
        for destroying it.  The caller does not take ownership of the zcert_t
        objects.
        """
        return utils.lib.zcertstore_certs(self._p)

    def state(self):
        """
        Return the state stored in certstore
        """
        return utils.lib.zcertstore_state(self._p)

    @staticmethod
    def test(verbose):
        """
        Self test of this class
        """
        utils.lib.zcertstore_test(verbose)

################################################################################
#  THIS FILE IS 100% GENERATED BY ZPROJECT; DO NOT EDIT EXCEPT EXPERIMENTALLY  #
#  Read the zproject/README.md for information about making permanent changes. #
################################################################################



################################################################################
#  THIS FILE IS 100% GENERATED BY ZPROJECT; DO NOT EDIT EXCEPT EXPERIMENTALLY  #
#  Read the zproject/README.md for information about making permanent changes. #
################################################################################
from . import utils
from . import destructors
libczmq_destructors = destructors.lib

class Ztrie(object):
    """
    simple trie for tokenizable strings
    """

    def __init__(self, delimiter):
        """
        Creates a new ztrie.
        """
        p = utils.lib.ztrie_new(delimiter._p)
        if p == utils.ffi.NULL:
            raise MemoryError("Could not allocate person")

        # ffi.gc returns a copy of the cdata object which will have the
        # destructor called when the Python object is GC'd:
        # https://cffi.readthedocs.org/en/latest/using.html#ffi-interface
        self._p = utils.ffi.gc(p, libczmq_destructors.ztrie_destroy_py)

    def insert_route(self, path, data, destroy_data_fn):
        """
        Inserts a new route into the tree and attaches the data. Returns -1
        if the route already exists, otherwise 0. This method takes ownership of
        the provided data if a destroy_data_fn is provided.
        """
        return utils.lib.ztrie_insert_route(self._p, utils.to_bytes(path), data._p, destroy_data_fn)

    def remove_route(self, path):
        """
        Removes a route from the trie and destroys its data. Returns -1 if the
        route does not exists, otherwise 0.
        the start of the list call zlist_first (). Advances the cursor.
        """
        return utils.lib.ztrie_remove_route(self._p, utils.to_bytes(path))

    def matches(self, path):
        """
        Returns true if the path matches a route in the tree, otherwise false.
        """
        return utils.lib.ztrie_matches(self._p, utils.to_bytes(path))

    def hit_data(self):
        """
        Returns the data of a matched route from last ztrie_matches. If the path
        did not match, returns NULL. Do not delete the data as it's owned by
        ztrie.
        """
        return utils.lib.ztrie_hit_data(self._p)

    def hit_parameter_count(self):
        """
        Returns the count of parameters that a matched route has.
        """
        return utils.lib.ztrie_hit_parameter_count(self._p)

    def hit_parameters(self):
        """
        Returns the parameters of a matched route with named regexes from last
        ztrie_matches. If the path did not match or the route did not contain any
        named regexes, returns NULL.
        """
        return utils.lib.ztrie_hit_parameters(self._p)

    def hit_asterisk_match(self):
        """
        Returns the asterisk matched part of a route, if there has been no match
        or no asterisk match, returns NULL.
        """
        return utils.lib.ztrie_hit_asterisk_match(self._p)

    def print_py(self):
        """
        Print the trie
        """
        utils.lib.ztrie_print(self._p)

    @staticmethod
    def test(verbose):
        """
        Self test of this class.
        """
        utils.lib.ztrie_test(verbose)

################################################################################
#  THIS FILE IS 100% GENERATED BY ZPROJECT; DO NOT EDIT EXCEPT EXPERIMENTALLY  #
#  Read the zproject/README.md for information about making permanent changes. #
################################################################################



################################################################################
#  THIS FILE IS 100% GENERATED BY ZPROJECT; DO NOT EDIT EXCEPT EXPERIMENTALLY  #
#  Read the zproject/README.md for information about making permanent changes. #
################################################################################
from . import utils
from . import destructors
libczmq_destructors = destructors.lib

class Zloop(object):
    """
    event-driven reactor
    """

    def __init__(self):
        """
        Create a new zloop reactor
        """
        p = utils.lib.zloop_new()
        if p == utils.ffi.NULL:
            raise MemoryError("Could not allocate person")

        # ffi.gc returns a copy of the cdata object which will have the
        # destructor called when the Python object is GC'd:
        # https://cffi.readthedocs.org/en/latest/using.html#ffi-interface
        self._p = utils.ffi.gc(p, libczmq_destructors.zloop_destroy_py)

    def reader(self, sock, handler, arg):
        """
        Register socket reader with the reactor. When the reader has messages,
        the reactor will call the handler, passing the arg. Returns 0 if OK, -1
        if there was an error. If you register the same socket more than once,
        each instance will invoke its corresponding handler.
        """
        return utils.lib.zloop_reader(self._p, sock._p, handler, arg._p)

    def reader_end(self, sock):
        """
        Cancel a socket reader from the reactor. If multiple readers exist for
        same socket, cancels ALL of them.
        """
        utils.lib.zloop_reader_end(self._p, sock._p)

    def reader_set_tolerant(self, sock):
        """
        Configure a registered reader to ignore errors. If you do not set this,
        then readers that have errors are removed from the reactor silently.
        """
        utils.lib.zloop_reader_set_tolerant(self._p, sock._p)

    def poller(self, item, handler, arg):
        """
        Register low-level libzmq pollitem with the reactor. When the pollitem
        is ready, will call the handler, passing the arg. Returns 0 if OK, -1
        if there was an error. If you register the pollitem more than once, each
        instance will invoke its corresponding handler. A pollitem with
        socket=NULL and fd=0 means 'poll on FD zero'.
        """
        return utils.lib.zloop_poller(self._p, item._p, handler, arg._p)

    def poller_end(self, item):
        """
        Cancel a pollitem from the reactor, specified by socket or FD. If both
        are specified, uses only socket. If multiple poll items exist for same
        socket/FD, cancels ALL of them.
        """
        utils.lib.zloop_poller_end(self._p, item._p)

    def poller_set_tolerant(self, item):
        """
        Configure a registered poller to ignore errors. If you do not set this,
        then poller that have errors are removed from the reactor silently.
        """
        utils.lib.zloop_poller_set_tolerant(self._p, item._p)

    def timer(self, delay, times, handler, arg):
        """
        Register a timer that expires after some delay and repeats some number of
        times. At each expiry, will call the handler, passing the arg. To run a
        timer forever, use 0 times. Returns a timer_id that is used to cancel the
        timer in the future. Returns -1 if there was an error.
        """
        return utils.lib.zloop_timer(self._p, delay, times, handler, arg._p)

    def timer_end(self, timer_id):
        """
        Cancel a specific timer identified by a specific timer_id (as returned by
        zloop_timer).
        """
        return utils.lib.zloop_timer_end(self._p, timer_id)

    def ticket(self, handler, arg):
        """
        Register a ticket timer. Ticket timers are very fast in the case where
        you use a lot of timers (thousands), and frequently remove and add them.
        The main use case is expiry timers for servers that handle many clients,
        and which reset the expiry timer for each message received from a client.
        Whereas normal timers perform poorly as the number of clients grows, the
        cost of ticket timers is constant, no matter the number of clients. You
        must set the ticket delay using zloop_set_ticket_delay before creating a
        ticket. Returns a handle to the timer that you should use in
        zloop_ticket_reset and zloop_ticket_delete.
        """
        return utils.lib.zloop_ticket(self._p, handler, arg._p)

    def ticket_reset(self, handle):
        """
        Reset a ticket timer, which moves it to the end of the ticket list and
        resets its execution time. This is a very fast operation.
        """
        utils.lib.zloop_ticket_reset(self._p, handle._p)

    def ticket_delete(self, handle):
        """
        Delete a ticket timer. We do not actually delete the ticket here, as
        other code may still refer to the ticket. We mark as deleted, and remove
        later and safely.
        """
        utils.lib.zloop_ticket_delete(self._p, handle._p)

    def set_ticket_delay(self, ticket_delay):
        """
        Set the ticket delay, which applies to all tickets. If you lower the
        delay and there are already tickets created, the results are undefined.
        """
        utils.lib.zloop_set_ticket_delay(self._p, ticket_delay)

    def set_max_timers(self, max_timers):
        """
        Set hard limit on number of timers allowed. Setting more than a small
        number of timers (10-100) can have a dramatic impact on the performance
        of the reactor. For high-volume cases, use ticket timers. If the hard
        limit is reached, the reactor stops creating new timers and logs an
        error.
        """
        utils.lib.zloop_set_max_timers(self._p, max_timers)

    def set_verbose(self, verbose):
        """
        Set verbose tracing of reactor on/off. The default verbose setting is
        off (false).
        """
        utils.lib.zloop_set_verbose(self._p, verbose)

    def set_nonstop(self, nonstop):
        """
        By default the reactor stops if the process receives a SIGINT or SIGTERM
        signal. This makes it impossible to shut-down message based architectures
        like zactors. This method lets you switch off break handling. The default
        nonstop setting is off (false).
        """
        utils.lib.zloop_set_nonstop(self._p, nonstop)

    def start(self):
        """
        Start the reactor. Takes control of the thread and returns when the 0MQ
        context is terminated or the process is interrupted, or any event handler
        returns -1. Event handlers may register new sockets and timers, and
        cancel sockets. Returns 0 if interrupted, -1 if canceled by a handler.
        """
        return utils.lib.zloop_start(self._p)

    @staticmethod
    def test(verbose):
        """
        Self test of this class.
        """
        utils.lib.zloop_test(verbose)

################################################################################
#  THIS FILE IS 100% GENERATED BY ZPROJECT; DO NOT EDIT EXCEPT EXPERIMENTALLY  #
#  Read the zproject/README.md for information about making permanent changes. #
################################################################################



################################################################################
#  THIS FILE IS 100% GENERATED BY ZPROJECT; DO NOT EDIT EXCEPT EXPERIMENTALLY  #
#  Read the zproject/README.md for information about making permanent changes. #
################################################################################
from __future__ import print_function
import os
import sys
from ctypes.util import find_library

import cffi
ffi = cffi.FFI()

try:
    # If LD_LIBRARY_PATH or your OSs equivalent is set, this is the only way to
    # load the library.  If we use find_library below, we get the wrong result.
    if os.name == 'posix':
        if sys.platform == 'darwin':
            libpath = 'libczmq.4.dylib'
        else:
            libpath = 'libczmq.so.4'
    elif os.name == 'nt':
        libpath = 'libczmq.dll'
    lib = ffi.dlopen(libpath)
except OSError:
    libpath = find_library("czmq")
    if not libpath:
        raise ImportError("Unable to find libczmq")
    lib = ffi.dlopen(libpath)

from czmq_cffi.cdefs import czmq_cdefs

for cdef in czmq_cdefs:
   ffi.cdef (cdef)



################################################################################
#  THIS FILE IS 100% GENERATED BY ZPROJECT; DO NOT EDIT EXCEPT EXPERIMENTALLY  #
#  Read the zproject/README.md for information about making permanent changes. #
################################################################################
from . import utils
from . import destructors
libczmq_destructors = destructors.lib

class ZhttpServerOptions(object):
    """
    zhttp server.
    """

    def __init__(self):
        """
        Create a new zhttp_server_options.
        """
        p = utils.lib.zhttp_server_options_new()
        if p == utils.ffi.NULL:
            raise MemoryError("Could not allocate person")

        # ffi.gc returns a copy of the cdata object which will have the
        # destructor called when the Python object is GC'd:
        # https://cffi.readthedocs.org/en/latest/using.html#ffi-interface
        self._p = utils.ffi.gc(p, libczmq_destructors.zhttp_server_options_destroy_py)

    @staticmethod
    def from_config(config):
        """
        Create options from config tree.
        """
        return utils.lib.zhttp_server_options_from_config(config._p)

    def port(self):
        """
        Get the server listening port.
        """
        return utils.lib.zhttp_server_options_port(self._p)

    def set_port(self, port):
        """
        Set the server listening port
        """
        utils.lib.zhttp_server_options_set_port(self._p, port)

    def backend_address(self):
        """
        Get the address sockets should connect to in order to receive requests.
        """
        return utils.lib.zhttp_server_options_backend_address(self._p)

    def set_backend_address(self, address):
        """
        Set the address sockets should connect to in order to receive requests.
        """
        utils.lib.zhttp_server_options_set_backend_address(self._p, utils.to_bytes(address))

    @staticmethod
    def test(verbose):
        """
        Self test of this class.
        """
        utils.lib.zhttp_server_options_test(verbose)

################################################################################
#  THIS FILE IS 100% GENERATED BY ZPROJECT; DO NOT EDIT EXCEPT EXPERIMENTALLY  #
#  Read the zproject/README.md for information about making permanent changes. #
################################################################################



################################################################################
#  THIS FILE IS 100% GENERATED BY ZPROJECT; DO NOT EDIT EXCEPT EXPERIMENTALLY  #
#  Read the zproject/README.md for information about making permanent changes. #
################################################################################
from . import utils
from . import destructors
libczmq_destructors = destructors.lib

class Zproc(object):
    """
    process configuration and status
    """

    def __init__(self):
        """
        Create a new zproc.
        NOTE: On Windows and with libzmq3 and libzmq2 this function
        returns NULL. Code needs to be ported there.
        """
        p = utils.lib.zproc_new()
        if p == utils.ffi.NULL:
            raise MemoryError("Could not allocate person")

        # ffi.gc returns a copy of the cdata object which will have the
        # destructor called when the Python object is GC'd:
        # https://cffi.readthedocs.org/en/latest/using.html#ffi-interface
        self._p = utils.ffi.gc(p, libczmq_destructors.zproc_destroy_py)

    def args(self):
        """
        Return command line arguments (the first item is the executable) or
        NULL if not set.
        """
        return utils.lib.zproc_args(self._p)

    def set_args(self, arguments):
        """
        Setup the command line arguments, the first item must be an (absolute) filename
        to run.
        """
        foo = utils.to_strings (argv_p)
        if foo is not None:
            foo_p = utils.ffi.new("struct _zlist_t *[1]")
            foo_p [0] = foo
            utils.lib.dproto_set_argv(self._p, foo_p)
            return

        utils.lib.zproc_set_args(self._p, utils.ffi.new("zlist_t **", arguments._p))

    def set_argsx(self, arguments, *arguments_args):
        """
        Setup the command line arguments, the first item must be an (absolute) filename
        to run. Variadic function, must be NULL terminated.
        """
        utils.lib.zproc_set_argsx(self._p, utils.to_bytes(arguments), *arguments_args)

    def set_env(self, arguments):
        """
        Setup the environment variables for the process.
        """
        utils.lib.zproc_set_env(self._p, utils.ffi.new("zhash_t **", arguments._p))

    def set_stdin(self, socket):
        """
        Connects process stdin with a readable ('>', connect) zeromq socket. If
        socket argument is NULL, zproc creates own managed pair of inproc
        sockets.  The writable one is then accessbile via zproc_stdin method.
        """
        utils.lib.zproc_set_stdin(self._p, socket._p)

    def set_stdout(self, socket):
        """
        Connects process stdout with a writable ('@', bind) zeromq socket. If
        socket argument is NULL, zproc creates own managed pair of inproc
        sockets.  The readable one is then accessbile via zproc_stdout method.
        """
        utils.lib.zproc_set_stdout(self._p, socket._p)

    def set_stderr(self, socket):
        """
        Connects process stderr with a writable ('@', bind) zeromq socket. If
        socket argument is NULL, zproc creates own managed pair of inproc
        sockets.  The readable one is then accessbile via zproc_stderr method.
        """
        utils.lib.zproc_set_stderr(self._p, socket._p)

    def stdin(self):
        """
        Return subprocess stdin writable socket. NULL for
        not initialized or external sockets.
        """
        return utils.lib.zproc_stdin(self._p)

    def stdout(self):
        """
        Return subprocess stdout readable socket. NULL for
        not initialized or external sockets.
        """
        return utils.lib.zproc_stdout(self._p)

    def stderr(self):
        """
        Return subprocess stderr readable socket. NULL for
        not initialized or external sockets.
        """
        return utils.lib.zproc_stderr(self._p)

    def run(self):
        """
        Starts the process, return just before execve/CreateProcess.
        """
        return utils.lib.zproc_run(self._p)

    def returncode(self):
        """
        process exit code
        """
        return utils.lib.zproc_returncode(self._p)

    def pid(self):
        """
        PID of the process
        """
        return utils.lib.zproc_pid(self._p)

    def running(self):
        """
        return true if process is running, false if not yet started or finished
        """
        return utils.lib.zproc_running(self._p)

    def wait(self, timeout):
        """
        The timeout should be zero or greater, or -1 to wait indefinitely.
        wait or poll process status, return return code
        """
        return utils.lib.zproc_wait(self._p, timeout)

    def shutdown(self, timeout):
        """
        send SIGTERM signal to the subprocess, wait for grace period and
        eventually send SIGKILL
        """
        utils.lib.zproc_shutdown(self._p, timeout)

    def actor(self):
        """
        return internal actor, useful for the polling if process died
        """
        return utils.lib.zproc_actor(self._p)

    def kill(self, signal):
        """
        send a signal to the subprocess
        """
        utils.lib.zproc_kill(self._p, signal)

    def set_verbose(self, verbose):
        """
        set verbose mode
        """
        utils.lib.zproc_set_verbose(self._p, verbose)

    @staticmethod
    def test(verbose):
        """
        Self test of this class.
        """
        utils.lib.zproc_test(verbose)

################################################################################
#  THIS FILE IS 100% GENERATED BY ZPROJECT; DO NOT EDIT EXCEPT EXPERIMENTALLY  #
#  Read the zproject/README.md for information about making permanent changes. #
################################################################################



################################################################################
#  THIS FILE IS 100% GENERATED BY ZPROJECT; DO NOT EDIT EXCEPT EXPERIMENTALLY  #
#  Read the zproject/README.md for information about making permanent changes. #
################################################################################
from . import utils
from . import destructors
libczmq_destructors = destructors.lib

class Zuuid(object):
    """
    UUID support class
    """

    def __init__(self):
        """
        Create a new UUID object.
        """
        p = utils.lib.zuuid_new()
        if p == utils.ffi.NULL:
            raise MemoryError("Could not allocate person")

        # ffi.gc returns a copy of the cdata object which will have the
        # destructor called when the Python object is GC'd:
        # https://cffi.readthedocs.org/en/latest/using.html#ffi-interface
        self._p = utils.ffi.gc(p, libczmq_destructors.zuuid_destroy_py)

    @staticmethod
    def new_from(source):
        """
        Create UUID object from supplied ZUUID_LEN-octet value.
        """
        return utils.lib.zuuid_new_from(source)

    def set(self, source):
        """
        Set UUID to new supplied ZUUID_LEN-octet value.
        """
        utils.lib.zuuid_set(self._p, source)

    def set_str(self, source):
        """
        Set UUID to new supplied string value skipping '-' and '{' '}'
        optional delimiters. Return 0 if OK, else returns -1.
        """
        return utils.lib.zuuid_set_str(self._p, utils.to_bytes(source))

    def data(self):
        """
        Return UUID binary data.
        """
        return utils.lib.zuuid_data(self._p)

    def size(self):
        """
        Return UUID binary size
        """
        return utils.lib.zuuid_size(self._p)

    def str(self):
        """
        Returns UUID as string
        """
        return utils.lib.zuuid_str(self._p)

    def str_canonical(self):
        """
        Return UUID in the canonical string format: 8-4-4-4-12, in lower
        case. Caller does not modify or free returned value. See
        http://en.wikipedia.org/wiki/Universally_unique_identifier
        """
        return utils.lib.zuuid_str_canonical(self._p)

    def export(self, target):
        """
        Store UUID blob in target array
        """
        utils.lib.zuuid_export(self._p, target)

    def eq(self, compare):
        """
        Check if UUID is same as supplied value
        """
        return utils.lib.zuuid_eq(self._p, compare)

    def neq(self, compare):
        """
        Check if UUID is different from supplied value
        """
        return utils.lib.zuuid_neq(self._p, compare)

    def dup(self):
        """
        Make copy of UUID object; if uuid is null, or memory was exhausted,
        returns null.
        """
        return utils.lib.zuuid_dup(self._p)

    @staticmethod
    def test(verbose):
        """
        Self test of this class.
        """
        utils.lib.zuuid_test(verbose)

################################################################################
#  THIS FILE IS 100% GENERATED BY ZPROJECT; DO NOT EDIT EXCEPT EXPERIMENTALLY  #
#  Read the zproject/README.md for information about making permanent changes. #
################################################################################



################################################################################
#  THIS FILE IS 100% GENERATED BY ZPROJECT; DO NOT EDIT EXCEPT EXPERIMENTALLY  #
#  Read the zproject/README.md for information about making permanent changes. #
################################################################################
from . import utils
from . import destructors
libczmq_destructors = destructors.lib

class Zosc(object):
    """
    Create and decode Open Sound Control messages. (OSC)

OSC is a serialisation format (and usually transported over UDP) which is
supported by many applications and appliances. It is a de facto protocol
for networking sound synthesizers, computers, and other multimedia devices
for purposes such as musical performance or show control. It is also often
used for rapid prototyping purposes due to the support by many applications
and frameworks in this field. With ZeroMQ's DGRAM sockets it is possible
to use ZeroMQ to send and receive OSC messages which can be understood by
any device supporting OSC.

Example creating an OSC message:

    zosc_t* conm = zosc_create("/someaddress", "iihfdscF",
                        1, 2, 3, 3.14, 6.283185307179586, "greetings", 'q');

Decoding a message:

    int rc = zosc_retr(oscmsg, "iihfdscF", &intx, &inty, &intz, &floatz,
                        &doublez, &strings, &charq, &someBool);

See the class's test method for more examples how to use the class.
    """

    def __init__(self, address):
        """
        Create a new empty OSC message with the specified address string.
        """
        p = utils.lib.zosc_new(utils.to_bytes(address))
        if p == utils.ffi.NULL:
            raise MemoryError("Could not allocate person")

        # ffi.gc returns a copy of the cdata object which will have the
        # destructor called when the Python object is GC'd:
        # https://cffi.readthedocs.org/en/latest/using.html#ffi-interface
        self._p = utils.ffi.gc(p, libczmq_destructors.zosc_destroy_py)

    @staticmethod
    def fromframe(frame):
        """
        Create a new OSC message from the specified zframe. Takes ownership of
        the zframe.
        """
        return utils.lib.zosc_fromframe(frame._p)

    @staticmethod
    def frommem(data, size):
        """
        Create a new zosc message from memory. Take ownership of the memory
        and calling free on the data after construction.
        """
        return utils.lib.zosc_frommem(data, size)

    @staticmethod
    def create(address, format, *format_args):
        """
        Create a new zosc message from the given format and arguments.
        The format type tags are as follows:
          i - 32bit integer
          h - 64bit integer
          f - 32bit floating point number (IEEE)
          d - 64bit (double) floating point number
          s - string (NULL terminated)
          t = timetag: an OSC timetag in NTP format (uint64_t)
          S - symbol
          c - char
          m - 4 byte midi packet (8 digits hexadecimal)
          T - TRUE (no value required)
          F - FALSE (no value required)
          N - NIL (no value required)
          I - Impulse (for triggers) or INFINITUM (no value required)
          b - binary blob
        """
        return utils.lib.zosc_create(utils.to_bytes(address), utils.to_bytes(format), *format_args)

    def size(self):
        """
        Return chunk data size
        """
        return utils.lib.zosc_size(self._p)

    def data(self):
        """
        Return OSC chunk data. Caller does not own the data!
        """
        return utils.lib.zosc_data(self._p)

    def address(self):
        """
        Return the OSC address string
        """
        return utils.lib.zosc_address(self._p)

    def format(self):
        """
        Return the OSC format of the message.
          i - 32bit integer
          h - 64bit integer
          f - 32bit floating point number (IEEE)
          d - 64bit (double) floating point number
          s - string (NULL terminated)
          t = timetag: an OSC timetag in NTP format (uint64_t)
          S - symbol
          c - char
          m - 4 byte midi packet (8 digits hexadecimal)
          T - TRUE (no value required)
          F - FALSE (no value required)
          N - NIL (no value required)
          I - Impulse (for triggers) or INFINITUM (no value required)
          b - binary blob
        """
        return utils.lib.zosc_format(self._p)

    def append(self, format, *format_args):
        """
        Append data to the osc message. The format describes the data that
        needs to be appended in the message. This essentially relocates all
        data!
        The format type tags are as follows:
          i - 32bit integer
          h - 64bit integer
          f - 32bit floating point number (IEEE)
          d - 64bit (double) floating point number
          s - string (NULL terminated)
          t = timetag: an OSC timetag in NTP format (uint64_t)
          S - symbol
          c - char
          m - 4 byte midi packet (8 digits hexadecimal)
          T - TRUE (no value required)
          F - FALSE (no value required)
          N - NIL (no value required)
          I - Impulse (for triggers) or INFINITUM (no value required)
          b - binary blob
        """
        return utils.lib.zosc_append(self._p, utils.to_bytes(format), *format_args)

    def retr(self, format, *format_args):
        """
        Retrieve the values provided by the given format. Note that zosc_retr
        creates the objects and the caller must destroy them when finished.
        The supplied pointers do not need to be initialized. Returns 0 if
        successful, or -1 if it failed to retrieve a value in which case the
        pointers are not modified. If an argument pointer is NULL is skips the
        value. See the format method for a detailed list op type tags for the
        format string.
        """
        return utils.lib.zosc_retr(self._p, utils.to_bytes(format), *format_args)

    def dup(self):
        """
        Create copy of the message, as new chunk object. Returns a fresh zosc_t
        object, or null if there was not enough heap memory. If chunk is null,
        returns null.
        """
        return utils.lib.zosc_dup(self._p)

    def pack(self):
        """
        Transform zosc into a zframe that can be sent in a message.
        """
        return utils.lib.zosc_pack(self._p)

    @staticmethod
    def packx(self_p):
        """
        Transform zosc into a zframe that can be sent in a message.
        Take ownership of the chunk.
        """
        return utils.lib.zosc_packx(utils.ffi.new("zosc_t **", self_p._p))

    @staticmethod
    def unpack(frame):
        """
        Transform a zframe into a zosc.
        """
        return utils.lib.zosc_unpack(frame._p)

    def print_py(self):
        """
        Dump OSC message to stdout, for debugging and tracing.
        """
        utils.lib.zosc_print(self._p)

    @staticmethod
    def is_py(self):
        """
        Probe the supplied object, and report if it looks like a zosc_t.
        """
        return utils.lib.zosc_is(self._p)

    def first(self, type):
        """
        Return a pointer to the item at the head of the OSC data.
        Sets the given char argument to the type tag of the data.
        If the message is empty, returns NULL and the sets the
        given char to NULL.
        """
        return utils.lib.zosc_first(self._p, utils.ffi.new("char_t **", type._p))

    def next(self, type):
        """
        Return the next item of the OSC message. If the list is empty, returns
        NULL. To move to the start of the OSC message call zosc_first ().
        """
        return utils.lib.zosc_next(self._p, utils.ffi.new("char_t **", type._p))

    def last(self, type):
        """
        Return a pointer to the item at the tail of the OSC message.
        Sets the given char argument to the type tag of the data. If
        the message is empty, returns NULL.
        """
        return utils.lib.zosc_last(self._p, utils.ffi.new("char_t **", type._p))

    def pop_int32(self, val):
        """
        Set the provided 32 bit integer from value at the current cursor position in the message.
        If the type tag at the current position does not correspond it will fail and
        return -1. Returns 0 on success.
        """
        return utils.lib.zosc_pop_int32(self._p, val)

    def pop_int64(self, val):
        """
        Set the provided 64 bit integer from the value at the current cursor position in the message.
        If the type tag at the current position does not correspond it will fail and
        return -1. Returns 0 on success.
        """
        return utils.lib.zosc_pop_int64(self._p, utils.ffi.new("msecs_t **", val._p))

    def pop_float(self, val):
        """
        Set the provided float from the value at the current cursor position in the message.
        If the type tag at the current position does not correspond it will fail and
        return -1. Returns 0 on success.
        """
        return utils.lib.zosc_pop_float(self._p, val)

    def pop_double(self, val):
        """
        Set the provided double from the value at the current cursor position in the message.
        If the type tag at the current position does not correspond it will fail and
        return -1. Returns 0 on success.
        """
        return utils.lib.zosc_pop_double(self._p, val)

    def pop_string(self, val):
        """
        Set the provided string from the value at the current cursor position in the message.
        If the type tag at the current position does not correspond it will fail and
        return -1. Returns 0 on success. Caller owns the string!
        """
        return utils.lib.zosc_pop_string(self._p, utils.to_bytes(val))

    def pop_char(self, val):
        """
        Set the provided char from the value at the current cursor position in the message.
        If the type tag at the current position does not correspond it will fail and
        return -1. Returns 0 on success.
        """
        return utils.lib.zosc_pop_char(self._p, utils.ffi.new("char_t **", val._p))

    def pop_bool(self, val):
        """
        Set the provided boolean from the type tag in the message. Booleans are not represented
        in the data in the message, only in the type tag. If the type tag at the current
        position does not correspond it will fail and return -1. Returns 0 on success.
        """
        return utils.lib.zosc_pop_bool(self._p, val)

    def pop_midi(self, val):
        """
        Set the provided 4 bytes (unsigned 32bit int) from the value at the current
        cursor position in the message. If the type tag at the current position does
        not correspond it will fail and return -1. Returns 0 on success.
        """
        return utils.lib.zosc_pop_midi(self._p, val)

    @staticmethod
    def test(verbose):
        """
        Self test of this class.
        """
        utils.lib.zosc_test(verbose)

################################################################################
#  THIS FILE IS 100% GENERATED BY ZPROJECT; DO NOT EDIT EXCEPT EXPERIMENTALLY  #
#  Read the zproject/README.md for information about making permanent changes. #
################################################################################



################################################################################
#  THIS FILE IS 100% GENERATED BY ZPROJECT; DO NOT EDIT EXCEPT EXPERIMENTALLY  #
#  Read the zproject/README.md for information about making permanent changes. #
################################################################################
from . import utils
from . import destructors
libczmq_destructors = destructors.lib

class Zchunk(object):
    """
    work with memory chunks
    """

    def __init__(self, data, size):
        """
        Create a new chunk of the specified size. If you specify the data, it
        is copied into the chunk. If you do not specify the data, the chunk is
        allocated and left empty, and you can then add data using zchunk_append.
        """
        p = utils.lib.zchunk_new(data, size)
        if p == utils.ffi.NULL:
            raise MemoryError("Could not allocate person")

        # ffi.gc returns a copy of the cdata object which will have the
        # destructor called when the Python object is GC'd:
        # https://cffi.readthedocs.org/en/latest/using.html#ffi-interface
        self._p = utils.ffi.gc(p, libczmq_destructors.zchunk_destroy_py)

    @staticmethod
    def frommem(data, size, destructor, hint):
        """
        Create a new chunk from memory. Take ownership of the memory and calling the destructor
        on destroy.
        """
        return utils.lib.zchunk_frommem(data, size, destructor, hint._p)

    def resize(self, size):
        """
        Resizes chunk max_size as requested; chunk_cur size is set to zero
        """
        utils.lib.zchunk_resize(self._p, size)

    def size(self):
        """
        Return chunk cur size
        """
        return utils.lib.zchunk_size(self._p)

    def max_size(self):
        """
        Return chunk max size
        """
        return utils.lib.zchunk_max_size(self._p)

    def data(self):
        """
        Return chunk data
        """
        return utils.lib.zchunk_data(self._p)

    def set(self, data, size):
        """
        Set chunk data from user-supplied data; truncate if too large. Data may
        be null. Returns actual size of chunk
        """
        return utils.lib.zchunk_set(self._p, data, size)

    def fill(self, filler, size):
        """
        Fill chunk data from user-supplied octet
        """
        return utils.lib.zchunk_fill(self._p, filler, size)

    def append(self, data, size):
        """
        Append user-supplied data to chunk, return resulting chunk size. If the
        data would exceeded the available space, it is truncated. If you want to
        grow the chunk to accommodate new data, use the zchunk_extend method.
        """
        return utils.lib.zchunk_append(self._p, data, size)

    def extend(self, data, size):
        """
        Append user-supplied data to chunk, return resulting chunk size. If the
        data would exceeded the available space, the chunk grows in size.
        """
        return utils.lib.zchunk_extend(self._p, data, size)

    def consume(self, source):
        """
        Copy as much data from 'source' into the chunk as possible; returns the
        new size of chunk. If all data from 'source' is used, returns exhausted
        on the source chunk. Source can be consumed as many times as needed until
        it is exhausted. If source was already exhausted, does not change chunk.
        """
        return utils.lib.zchunk_consume(self._p, source._p)

    def exhausted(self):
        """
        Returns true if the chunk was exhausted by consume methods, or if the
        chunk has a size of zero.
        """
        return utils.lib.zchunk_exhausted(self._p)

    @staticmethod
    def read(handle, bytes):
        """
        Read chunk from an open file descriptor
        """
        return utils.lib.zchunk_read(handle, bytes)

    def write(self, handle):
        """
        Write chunk to an open file descriptor
        """
        return utils.lib.zchunk_write(self._p, handle)

    @staticmethod
    def slurp(filename, maxsize):
        """
        Try to slurp an entire file into a chunk. Will read up to maxsize of
        the file. If maxsize is 0, will attempt to read the entire file and
        fail with an assertion if that cannot fit into memory. Returns a new
        chunk containing the file data, or NULL if the file could not be read.
        """
        return utils.lib.zchunk_slurp(utils.to_bytes(filename), maxsize)

    def dup(self):
        """
        Create copy of chunk, as new chunk object. Returns a fresh zchunk_t
        object, or null if there was not enough heap memory. If chunk is null,
        returns null.
        """
        return utils.lib.zchunk_dup(self._p)

    def strhex(self):
        """
        Return chunk data encoded as printable hex string. Caller must free
        string when finished with it.
        """
        return utils.lib.zchunk_strhex(self._p)

    def strdup(self):
        """
        Return chunk data copied into freshly allocated string
        Caller must free string when finished with it.
        """
        return utils.lib.zchunk_strdup(self._p)

    def streq(self, string):
        """
        Return TRUE if chunk body is equal to string, excluding terminator
        """
        return utils.lib.zchunk_streq(self._p, utils.to_bytes(string))

    def pack(self):
        """
        Transform zchunk into a zframe that can be sent in a message.
        """
        return utils.lib.zchunk_pack(self._p)

    @staticmethod
    def packx(self_p):
        """
        Transform zchunk into a zframe that can be sent in a message.
        Take ownership of the chunk.
        """
        return utils.lib.zchunk_packx(utils.ffi.new("zchunk_t **", self_p._p))

    @staticmethod
    def unpack(frame):
        """
        Transform a zframe into a zchunk.
        """
        return utils.lib.zchunk_unpack(frame._p)

    def digest(self):
        """
        Calculate SHA1 digest for chunk, using zdigest class.
        """
        return utils.lib.zchunk_digest(self._p)

    def fprint(self, file):
        """
        Dump chunk to FILE stream, for debugging and tracing.
        """
        utils.lib.zchunk_fprint(self._p, file)

    def print_py(self):
        """
        Dump message to stderr, for debugging and tracing.
        See zchunk_fprint for details
        """
        utils.lib.zchunk_print(self._p)

    @staticmethod
    def is_py(self):
        """
        Probe the supplied object, and report if it looks like a zchunk_t.
        """
        return utils.lib.zchunk_is(self._p)

    @staticmethod
    def test(verbose):
        """
        Self test of this class.
        """
        utils.lib.zchunk_test(verbose)

################################################################################
#  THIS FILE IS 100% GENERATED BY ZPROJECT; DO NOT EDIT EXCEPT EXPERIMENTALLY  #
#  Read the zproject/README.md for information about making permanent changes. #
################################################################################



################################################################################
#  THIS FILE IS 100% GENERATED BY ZPROJECT; DO NOT EDIT EXCEPT EXPERIMENTALLY  #
#  Read the zproject/README.md for information about making permanent changes. #
################################################################################
from . import utils
from . import destructors
libczmq_destructors = destructors.lib

class Zconfig(object):
    """
    work with config files written in rfc.zeromq.org/spec:4/ZPL.
    """

    def __init__(self, name, parent):
        """
        Create new config item
        """
        p = utils.lib.zconfig_new(utils.to_bytes(name), parent._p)
        if p == utils.ffi.NULL:
            raise MemoryError("Could not allocate person")

        # ffi.gc returns a copy of the cdata object which will have the
        # destructor called when the Python object is GC'd:
        # https://cffi.readthedocs.org/en/latest/using.html#ffi-interface
        self._p = utils.ffi.gc(p, libczmq_destructors.zconfig_destroy_py)

    @staticmethod
    def load(filename):
        """
        Load a config tree from a specified ZPL text file; returns a zconfig_t
        reference for the root, if the file exists and is readable. Returns NULL
        if the file does not exist.
        """
        return utils.lib.zconfig_load(utils.to_bytes(filename))

    @staticmethod
    def loadf(format, *format_args):
        """
        Equivalent to zconfig_load, taking a format string instead of a fixed
        filename.
        """
        return utils.lib.zconfig_loadf(format, *format_args)

    def dup(self):
        """
        Create copy of zconfig, caller MUST free the value
        Create copy of config, as new zconfig object. Returns a fresh zconfig_t
        object. If config is null, or memory was exhausted, returns null.
        """
        return utils.lib.zconfig_dup(self._p)

    def name(self):
        """
        Return name of config item
        """
        return utils.lib.zconfig_name(self._p)

    def value(self):
        """
        Return value of config item
        """
        return utils.lib.zconfig_value(self._p)

    def put(self, path, value):
        """
        Insert or update configuration key with value
        """
        utils.lib.zconfig_put(self._p, utils.to_bytes(path), utils.to_bytes(value))

    def putf(self, path, format, *format_args):
        """
        Equivalent to zconfig_put, accepting a format specifier and variable
        argument list, instead of a single string value.
        """
        utils.lib.zconfig_putf(self._p, utils.to_bytes(path), format, *format_args)

    def get(self, path, default_value):
        """
        Get value for config item into a string value; leading slash is optional
        and ignored.
        """
        return utils.lib.zconfig_get(self._p, utils.to_bytes(path), utils.to_bytes(default_value))

    def set_name(self, name):
        """
        Set config item name, name may be NULL
        """
        utils.lib.zconfig_set_name(self._p, utils.to_bytes(name))

    def set_value(self, format, *format_args):
        """
        Set new value for config item. The new value may be a string, a printf
        format, or NULL. Note that if string may possibly contain '%', or if it
        comes from an insecure source, you must use '%s' as the format, followed
        by the string.
        """
        utils.lib.zconfig_set_value(self._p, format, *format_args)

    def child(self):
        """
        Find our first child, if any
        """
        return utils.lib.zconfig_child(self._p)

    def next(self):
        """
        Find our first sibling, if any
        """
        return utils.lib.zconfig_next(self._p)

    def locate(self, path):
        """
        Find a config item along a path; leading slash is optional and ignored.
        """
        return utils.lib.zconfig_locate(self._p, utils.to_bytes(path))

    def at_depth(self, level):
        """
        Locate the last config item at a specified depth
        """
        return utils.lib.zconfig_at_depth(self._p, level)

    def execute(self, handler, arg):
        """
        Execute a callback for each config item in the tree; returns zero if
        successful, else -1.
        """
        return utils.lib.zconfig_execute(self._p, handler, arg._p)

    def set_comment(self, format, *format_args):
        """
        Add comment to config item before saving to disk. You can add as many
        comment lines as you like. If you use a null format, all comments are
        deleted.
        """
        utils.lib.zconfig_set_comment(self._p, format, *format_args)

    def comments(self):
        """
        Return comments of config item, as zlist.
        """
        return utils.lib.zconfig_comments(self._p)

    def save(self, filename):
        """
        Save a config tree to a specified ZPL text file, where a filename
        "-" means dump to standard output.
        """
        return utils.lib.zconfig_save(self._p, utils.to_bytes(filename))

    def savef(self, format, *format_args):
        """
        Equivalent to zconfig_save, taking a format string instead of a fixed
        filename.
        """
        return utils.lib.zconfig_savef(self._p, format, *format_args)

    def filename(self):
        """
        Report filename used during zconfig_load, or NULL if none
        """
        return utils.lib.zconfig_filename(self._p)

    @staticmethod
    def reload(self_p):
        """
        Reload config tree from same file that it was previously loaded from.
        Returns 0 if OK, -1 if there was an error (and then does not change
        existing data).
        """
        return utils.lib.zconfig_reload(utils.ffi.new("zconfig_t **", self_p._p))

    @staticmethod
    def chunk_load(chunk):
        """
        Load a config tree from a memory chunk
        """
        return utils.lib.zconfig_chunk_load(chunk._p)

    def chunk_save(self):
        """
        Save a config tree to a new memory chunk
        """
        return utils.lib.zconfig_chunk_save(self._p)

    @staticmethod
    def str_load(string):
        """
        Load a config tree from a null-terminated string
        """
        return utils.lib.zconfig_str_load(utils.to_bytes(string))

    def str_save(self):
        """
        Save a config tree to a new null terminated string
        """
        return utils.lib.zconfig_str_save(self._p)

    def has_changed(self):
        """
        Return true if a configuration tree was loaded from a file and that
        file has changed in since the tree was loaded.
        """
        return utils.lib.zconfig_has_changed(self._p)

    def remove_subtree(self):
        """
        Destroy subtree (all children)
        """
        utils.lib.zconfig_remove_subtree(self._p)

    @staticmethod
    def remove(self_p):
        """
        Destroy node and subtree (all children)
        """
        utils.lib.zconfig_remove(utils.ffi.new("zconfig_t **", self_p._p))

    def fprint(self, file):
        """
        Print the config file to open stream
        """
        utils.lib.zconfig_fprint(self._p, file)

    def print_py(self):
        """
        Print properties of object
        """
        utils.lib.zconfig_print(self._p)

    @staticmethod
    def test(verbose):
        """
        Self test of this class
        """
        utils.lib.zconfig_test(verbose)

################################################################################
#  THIS FILE IS 100% GENERATED BY ZPROJECT; DO NOT EDIT EXCEPT EXPERIMENTALLY  #
#  Read the zproject/README.md for information about making permanent changes. #
################################################################################



################################################################################
#  THIS FILE IS 100% GENERATED BY ZPROJECT; DO NOT EDIT EXCEPT EXPERIMENTALLY  #
#  Read the zproject/README.md for information about making permanent changes. #
################################################################################
from . import utils
from . import destructors
libczmq_destructors = destructors.lib

class Zstr(object):
    """
    sending and receiving strings
    """

    @staticmethod
    def recv(source):
        """
        Receive C string from socket. Caller must free returned string using
        zstr_free(). Returns NULL if the context is being terminated or the
        process was interrupted.
        """
        return utils.lib.zstr_recv(source._p)

    @staticmethod
    def recvx(source, string_p, *string_p_args):
        """
        Receive a series of strings (until NULL) from multipart data.
        Each string is allocated and filled with string data; if there
        are not enough frames, unallocated strings are set to NULL.
        Returns -1 if the message could not be read, else returns the
        number of strings filled, zero or more. Free each returned string
        using zstr_free(). If not enough strings are provided, remaining
        multipart frames in the message are dropped.
        """
        return utils.lib.zstr_recvx(source._p, utils.to_bytes(string_p), *string_p_args)

    @staticmethod
    def recv_compress(source):
        """
        De-compress and receive C string from socket, received as a message
        with two frames: size of the uncompressed string, and the string itself.
        Caller must free returned string using zstr_free(). Returns NULL if the
        context is being terminated or the process was interrupted.
        """
        return utils.lib.zstr_recv_compress(source._p)

    @staticmethod
    def send(dest, string):
        """
        Send a C string to a socket, as a frame. The string is sent without
        trailing null byte; to read this you can use zstr_recv, or a similar
        method that adds a null terminator on the received string. String
        may be NULL, which is sent as "".
        """
        return utils.lib.zstr_send(dest._p, utils.to_bytes(string))

    @staticmethod
    def sendm(dest, string):
        """
        Send a C string to a socket, as zstr_send(), with a MORE flag, so that
        you can send further strings in the same multi-part message.
        """
        return utils.lib.zstr_sendm(dest._p, utils.to_bytes(string))

    @staticmethod
    def sendf(dest, format, *format_args):
        """
        Send a formatted string to a socket. Note that you should NOT use
        user-supplied strings in the format (they may contain '%' which
        will create security holes).
        """
        return utils.lib.zstr_sendf(dest._p, format, *format_args)

    @staticmethod
    def sendfm(dest, format, *format_args):
        """
        Send a formatted string to a socket, as for zstr_sendf(), with a
        MORE flag, so that you can send further strings in the same multi-part
        message.
        """
        return utils.lib.zstr_sendfm(dest._p, format, *format_args)

    @staticmethod
    def sendx(dest, string, *string_args):
        """
        Send a series of strings (until NULL) as multipart data
        Returns 0 if the strings could be sent OK, or -1 on error.
        """
        return utils.lib.zstr_sendx(dest._p, utils.to_bytes(string), *string_args)

    @staticmethod
    def send_compress(dest, string):
        """
        Compress and send a C string to a socket, as a message with two frames:
        size of the uncompressed string, and the string itself. The string is
        sent without trailing null byte; to read this you can use
        zstr_recv_compress, or a similar method that de-compresses and adds a
        null terminator on the received string.
        """
        return utils.lib.zstr_send_compress(dest._p, utils.to_bytes(string))

    @staticmethod
    def sendm_compress(dest, string):
        """
        Compress and send a C string to a socket, as zstr_send_compress(),
        with a MORE flag, so that you can send further strings in the same
        multi-part message.
        """
        return utils.lib.zstr_sendm_compress(dest._p, utils.to_bytes(string))

    @staticmethod
    def str(source):
        """
        Accepts a void pointer and returns a fresh character string. If source
        is null, returns an empty string.
        """
        return utils.lib.zstr_str(source._p)

    @staticmethod
    def free(string_p):
        """
        Free a provided string, and nullify the parent pointer. Safe to call on
        a null pointer.
        """
        utils.lib.zstr_free(utils.to_bytes(string_p))

    @staticmethod
    def test(verbose):
        """
        Self test of this class.
        """
        utils.lib.zstr_test(verbose)

################################################################################
#  THIS FILE IS 100% GENERATED BY ZPROJECT; DO NOT EDIT EXCEPT EXPERIMENTALLY  #
#  Read the zproject/README.md for information about making permanent changes. #
################################################################################



################################################################################
#  THIS FILE IS 100% GENERATED BY ZPROJECT; DO NOT EDIT EXCEPT EXPERIMENTALLY  #
#  Read the zproject/README.md for information about making permanent changes. #
################################################################################
from . import utils
from . import destructors
libczmq_destructors = destructors.lib

class Zclock(object):
    """
    millisecond clocks and delays
    """

    @staticmethod
    def sleep(msecs):
        """
        Sleep for a number of milliseconds
        """
        utils.lib.zclock_sleep(msecs)

    @staticmethod
    def time():
        """
        Return current system clock as milliseconds. Note that this clock can
        jump backwards (if the system clock is changed) so is unsafe to use for
        timers and time offsets. Use zclock_mono for that instead.
        """
        return utils.lib.zclock_time()

    @staticmethod
    def mono():
        """
        Return current monotonic clock in milliseconds. Use this when you compute
        time offsets. The monotonic clock is not affected by system changes and
        so will never be reset backwards, unlike a system clock.
        """
        return utils.lib.zclock_mono()

    @staticmethod
    def usecs():
        """
        Return current monotonic clock in microseconds. Use this when you compute
        time offsets. The monotonic clock is not affected by system changes and
        so will never be reset backwards, unlike a system clock.
        """
        return utils.lib.zclock_usecs()

    @staticmethod
    def timestr():
        """
        Return formatted date/time as fresh string. Free using zstr_free().
        """
        return utils.lib.zclock_timestr()

    @staticmethod
    def test(verbose):
        """
        Self test of this class.
        """
        utils.lib.zclock_test(verbose)

################################################################################
#  THIS FILE IS 100% GENERATED BY ZPROJECT; DO NOT EDIT EXCEPT EXPERIMENTALLY  #
#  Read the zproject/README.md for information about making permanent changes. #
################################################################################



################################################################################
#  THIS FILE IS 100% GENERATED BY ZPROJECT; DO NOT EDIT EXCEPT EXPERIMENTALLY  #
#  Read the zproject/README.md for information about making permanent changes. #
################################################################################
from . import utils
from . import destructors
libczmq_destructors = destructors.lib

class ZhttpServerConnection(object):
    """

    """

    def test(verbose):
        """
        Self test of this class.
        """
        utils.lib.zhttp_server_connection_test(verbose)

################################################################################
#  THIS FILE IS 100% GENERATED BY ZPROJECT; DO NOT EDIT EXCEPT EXPERIMENTALLY  #
#  Read the zproject/README.md for information about making permanent changes. #
################################################################################



################################################################################
#  THIS FILE IS 100% GENERATED BY ZPROJECT; DO NOT EDIT EXCEPT EXPERIMENTALLY  #
#  Read the zproject/README.md for information about making permanent changes. #
################################################################################
from . import utils
from . import destructors
libczmq_destructors = destructors.lib

class Zmsg(object):
    """
    working with multipart messages
    """

    def __init__(self):
        """
        Create a new empty message object
        """
        p = utils.lib.zmsg_new()
        if p == utils.ffi.NULL:
            raise MemoryError("Could not allocate person")

        # ffi.gc returns a copy of the cdata object which will have the
        # destructor called when the Python object is GC'd:
        # https://cffi.readthedocs.org/en/latest/using.html#ffi-interface
        self._p = utils.ffi.gc(p, libczmq_destructors.zmsg_destroy_py)

    @staticmethod
    def recv(source):
        """
        Receive message from socket, returns zmsg_t object or NULL if the recv
        was interrupted. Does a blocking recv. If you want to not block then use
        the zloop class or zmsg_recv_nowait or zmq_poll to check for socket input
        before receiving.
        """
        return utils.lib.zmsg_recv(source._p)

    @staticmethod
    def load(file):
        """
        Load/append an open file into new message, return the message.
        Returns NULL if the message could not be loaded.
        """
        return utils.lib.zmsg_load(file)

    @staticmethod
    def decode(frame):
        """
        Decodes a serialized message frame created by zmsg_encode () and returns
        a new zmsg_t object. Returns NULL if the frame was badly formatted or
        there was insufficient memory to work.
        """
        return utils.lib.zmsg_decode(frame._p)

    @staticmethod
    def new_signal(status):
        """
        Generate a signal message encoding the given status. A signal is a short
        message carrying a 1-byte success/failure code (by convention, 0 means
        OK). Signals are encoded to be distinguishable from "normal" messages.
        """
        return utils.lib.zmsg_new_signal(status)

    @staticmethod
    def send(self_p, dest):
        """
        Send message to destination socket, and destroy the message after sending
        it successfully. If the message has no frames, sends nothing but destroys
        the message anyhow. Nullifies the caller's reference to the message (as
        it is a destructor).
        """
        return utils.lib.zmsg_send(utils.ffi.new("zmsg_t **", self_p._p), dest._p)

    @staticmethod
    def sendm(self_p, dest):
        """
        Send message to destination socket as part of a multipart sequence, and
        destroy the message after sending it successfully. Note that after a
        zmsg_sendm, you must call zmsg_send or another method that sends a final
        message part. If the message has no frames, sends nothing but destroys
        the message anyhow. Nullifies the caller's reference to the message (as
        it is a destructor).
        """
        return utils.lib.zmsg_sendm(utils.ffi.new("zmsg_t **", self_p._p), dest._p)

    def size(self):
        """
        Return size of message, i.e. number of frames (0 or more).
        """
        return utils.lib.zmsg_size(self._p)

    def content_size(self):
        """
        Return total size of all frames in message.
        """
        return utils.lib.zmsg_content_size(self._p)

    def routing_id(self):
        """
        Return message routing ID, if the message came from a ZMQ_SERVER socket.
        Else returns zero.
        """
        return utils.lib.zmsg_routing_id(self._p)

    def set_routing_id(self, routing_id):
        """
        Set routing ID on message. This is used if/when the message is sent to a
        ZMQ_SERVER socket.
        """
        utils.lib.zmsg_set_routing_id(self._p, routing_id)

    def prepend(self, frame_p):
        """
        Push frame to the front of the message, i.e. before all other frames.
        Message takes ownership of frame, will destroy it when message is sent.
        Returns 0 on success, -1 on error. Deprecates zmsg_push, which did not
        nullify the caller's frame reference.
        """
        return utils.lib.zmsg_prepend(self._p, utils.ffi.new("zframe_t **", frame_p._p))

    def append(self, frame_p):
        """
        Add frame to the end of the message, i.e. after all other frames.
        Message takes ownership of frame, will destroy it when message is sent.
        Returns 0 on success. Deprecates zmsg_add, which did not nullify the
        caller's frame reference.
        """
        return utils.lib.zmsg_append(self._p, utils.ffi.new("zframe_t **", frame_p._p))

    def pop(self):
        """
        Remove first frame from message, if any. Returns frame, or NULL.
        """
        return utils.lib.zmsg_pop(self._p)

    def pushmem(self, data, size):
        """
        Push block of memory to front of message, as a new frame.
        Returns 0 on success, -1 on error.
        """
        return utils.lib.zmsg_pushmem(self._p, data, size)

    def addmem(self, data, size):
        """
        Add block of memory to the end of the message, as a new frame.
        Returns 0 on success, -1 on error.
        """
        return utils.lib.zmsg_addmem(self._p, data, size)

    def pushstr(self, string):
        """
        Push string as new frame to front of message.
        Returns 0 on success, -1 on error.
        """
        return utils.lib.zmsg_pushstr(self._p, utils.to_bytes(string))

    def addstr(self, string):
        """
        Push string as new frame to end of message.
        Returns 0 on success, -1 on error.
        """
        return utils.lib.zmsg_addstr(self._p, utils.to_bytes(string))

    def pushstrf(self, format, *format_args):
        """
        Push formatted string as new frame to front of message.
        Returns 0 on success, -1 on error.
        """
        return utils.lib.zmsg_pushstrf(self._p, format, *format_args)

    def addstrf(self, format, *format_args):
        """
        Push formatted string as new frame to end of message.
        Returns 0 on success, -1 on error.
        """
        return utils.lib.zmsg_addstrf(self._p, format, *format_args)

    def popstr(self):
        """
        Pop frame off front of message, return as fresh string. If there were
        no more frames in the message, returns NULL.
        """
        return utils.lib.zmsg_popstr(self._p)

    def addmsg(self, msg_p):
        """
        Push encoded message as a new frame. Message takes ownership of
        submessage, so the original is destroyed in this call. Returns 0 on
        success, -1 on error.
        """
        return utils.lib.zmsg_addmsg(self._p, utils.ffi.new("zmsg_t **", msg_p._p))

    def popmsg(self):
        """
        Remove first submessage from message, if any. Returns zmsg_t, or NULL if
        decoding was not successful.
        """
        return utils.lib.zmsg_popmsg(self._p)

    def remove(self, frame):
        """
        Remove specified frame from list, if present. Does not destroy frame.
        """
        utils.lib.zmsg_remove(self._p, frame._p)

    def first(self):
        """
        Set cursor to first frame in message. Returns frame, or NULL, if the
        message is empty. Use this to navigate the frames as a list.
        """
        return utils.lib.zmsg_first(self._p)

    def next(self):
        """
        Return the next frame. If there are no more frames, returns NULL. To move
        to the first frame call zmsg_first(). Advances the cursor.
        """
        return utils.lib.zmsg_next(self._p)

    def last(self):
        """
        Return the last frame. If there are no frames, returns NULL.
        """
        return utils.lib.zmsg_last(self._p)

    def save(self, file):
        """
        Save message to an open file, return 0 if OK, else -1. The message is
        saved as a series of frames, each with length and data. Note that the
        file is NOT guaranteed to be portable between operating systems, not
        versions of CZMQ. The file format is at present undocumented and liable
        to arbitrary change.
        """
        return utils.lib.zmsg_save(self._p, file)

    def encode(self):
        """
        Serialize multipart message to a single message frame. Use this method
        to send structured messages across transports that do not support
        multipart data. Allocates and returns a new frame containing the
        serialized message. To decode a serialized message frame, use
        zmsg_decode ().
        """
        return utils.lib.zmsg_encode(self._p)

    def dup(self):
        """
        Create copy of message, as new message object. Returns a fresh zmsg_t
        object. If message is null, or memory was exhausted, returns null.
        """
        return utils.lib.zmsg_dup(self._p)

    def print_py(self):
        """
        Send message to zsys log sink (may be stdout, or system facility as
        configured by zsys_set_logstream).
        Long messages are truncated.
        """
        utils.lib.zmsg_print(self._p)

    def print_n(self, size):
        """
        Send message to zsys log sink (may be stdout, or system facility as
        configured by zsys_set_logstream).
        Message length is specified; no truncation unless length is zero.
        Backwards compatible with zframe_print when length is zero.
        """
        utils.lib.zmsg_print_n(self._p, size)

    def eq(self, other):
        """
        Return true if the two messages have the same number of frames and each
        frame in the first message is identical to the corresponding frame in the
        other message. As with zframe_eq, return false if either message is NULL.
        """
        return utils.lib.zmsg_eq(self._p, other._p)

    def signal(self):
        """
        Return signal value, 0 or greater, if message is a signal, -1 if not.
        """
        return utils.lib.zmsg_signal(self._p)

    @staticmethod
    def is_py(self):
        """
        Probe the supplied object, and report if it looks like a zmsg_t.
        """
        return utils.lib.zmsg_is(self._p)

    @staticmethod
    def test(verbose):
        """
        Self test of this class.
        """
        utils.lib.zmsg_test(verbose)

################################################################################
#  THIS FILE IS 100% GENERATED BY ZPROJECT; DO NOT EDIT EXCEPT EXPERIMENTALLY  #
#  Read the zproject/README.md for information about making permanent changes. #
################################################################################



################################################################################
#  THIS FILE IS 100% GENERATED BY ZPROJECT; DO NOT EDIT EXCEPT EXPERIMENTALLY  #
#  Read the zproject/README.md for information about making permanent changes. #
################################################################################
from . import utils
from . import destructors
libczmq_destructors = destructors.lib

class Zactor(object):
    """
    provides a simple actor framework
    """

    def __init__(self, task, args):
        """
        Create a new actor passing arbitrary arguments reference.
        """
        p = utils.lib.zactor_new(task, args._p)
        if p == utils.ffi.NULL:
            raise MemoryError("Could not allocate person")

        # ffi.gc returns a copy of the cdata object which will have the
        # destructor called when the Python object is GC'd:
        # https://cffi.readthedocs.org/en/latest/using.html#ffi-interface
        self._p = utils.ffi.gc(p, libczmq_destructors.zactor_destroy_py)

    def send(self, msg_p):
        """
        Send a zmsg message to the actor, take ownership of the message
        and destroy when it has been sent.
        """
        return utils.lib.zactor_send(self._p, utils.ffi.new("zmsg_t **", msg_p._p))

    def recv(self):
        """
        Receive a zmsg message from the actor. Returns NULL if the actor
        was interrupted before the message could be received, or if there
        was a timeout on the actor.
        """
        return utils.lib.zactor_recv(self._p)

    @staticmethod
    def is_py(self):
        """
        Probe the supplied object, and report if it looks like a zactor_t.
        """
        return utils.lib.zactor_is(self._p)

    @staticmethod
    def resolve(self):
        """
        Probe the supplied reference. If it looks like a zactor_t instance,
        return the underlying libzmq actor handle; else if it looks like
        a libzmq actor handle, return the supplied value.
        """
        return utils.lib.zactor_resolve(self._p)

    def sock(self):
        """
        Return the actor's zsock handle. Use this when you absolutely need
        to work with the zsock instance rather than the actor.
        """
        return utils.lib.zactor_sock(self._p)

    def set_destructor(self, destructor):
        """
        Change default destructor by custom function. Actor MUST be able to handle new message instead of default $TERM.
        """
        utils.lib.zactor_set_destructor(self._p, destructor)

    @staticmethod
    def test(verbose):
        """
        Self test of this class.
        """
        utils.lib.zactor_test(verbose)

################################################################################
#  THIS FILE IS 100% GENERATED BY ZPROJECT; DO NOT EDIT EXCEPT EXPERIMENTALLY  #
#  Read the zproject/README.md for information about making permanent changes. #
################################################################################



################################################################################
#  THIS FILE IS 100% GENERATED BY ZPROJECT; DO NOT EDIT EXCEPT EXPERIMENTALLY  #
#  Read the zproject/README.md for information about making permanent changes. #
################################################################################
from . import utils
from . import destructors
libczmq_destructors = destructors.lib

class Ztimerset(object):
    """
    timer set
    """

    def __init__(self):
        """
        Create new timer set.
        """
        p = utils.lib.ztimerset_new()
        if p == utils.ffi.NULL:
            raise MemoryError("Could not allocate person")

        # ffi.gc returns a copy of the cdata object which will have the
        # destructor called when the Python object is GC'd:
        # https://cffi.readthedocs.org/en/latest/using.html#ffi-interface
        self._p = utils.ffi.gc(p, libczmq_destructors.ztimerset_destroy_py)

    def add(self, interval, handler, arg):
        """
        Add a timer to the set. Returns timer id if OK, -1 on failure.
        """
        return utils.lib.ztimerset_add(self._p, interval, handler, arg._p)

    def cancel(self, timer_id):
        """
        Cancel a timer. Returns 0 if OK, -1 on failure.
        """
        return utils.lib.ztimerset_cancel(self._p, timer_id)

    def set_interval(self, timer_id, interval):
        """
        Set timer interval. Returns 0 if OK, -1 on failure.
        This method is slow, canceling the timer and adding a new one yield better performance.
        """
        return utils.lib.ztimerset_set_interval(self._p, timer_id, interval)

    def reset(self, timer_id):
        """
        Reset timer to start interval counting from current time. Returns 0 if OK, -1 on failure.
        This method is slow, canceling the timer and adding a new one yield better performance.
        """
        return utils.lib.ztimerset_reset(self._p, timer_id)

    def timeout(self):
        """
        Return the time until the next interval.
        Should be used as timeout parameter for the zpoller wait method.
        The timeout is in msec.
        """
        return utils.lib.ztimerset_timeout(self._p)

    def execute(self):
        """
        Invoke callback function of all timers which their interval has elapsed.
        Should be call after zpoller wait method.
        Returns 0 if OK, -1 on failure.
        """
        return utils.lib.ztimerset_execute(self._p)

    @staticmethod
    def test(verbose):
        """
        Self test of this class.
        """
        utils.lib.ztimerset_test(verbose)

################################################################################
#  THIS FILE IS 100% GENERATED BY ZPROJECT; DO NOT EDIT EXCEPT EXPERIMENTALLY  #
#  Read the zproject/README.md for information about making permanent changes. #
################################################################################



################################################################################
#  THIS FILE IS 100% GENERATED BY ZPROJECT; DO NOT EDIT EXCEPT EXPERIMENTALLY  #
#  Read the zproject/README.md for information about making permanent changes. #
################################################################################
from . import utils
from . import destructors
libczmq_destructors = destructors.lib

class ZhttpRequest(object):
    """
    Http request that can be received from zhttp_server or sent to zhttp_client.
Class can be reused between send & recv calls.
Headers and Content is being destroyed after every send call.
    """

    def __init__(self):
        """
        Create a new http request.
        """
        p = utils.lib.zhttp_request_new()
        if p == utils.ffi.NULL:
            raise MemoryError("Could not allocate person")

        # ffi.gc returns a copy of the cdata object which will have the
        # destructor called when the Python object is GC'd:
        # https://cffi.readthedocs.org/en/latest/using.html#ffi-interface
        self._p = utils.ffi.gc(p, libczmq_destructors.zhttp_request_destroy_py)

    def recv(self, sock):
        """
        Receive a new request from zhttp_server.
        Return the underlying connection if successful, to be used when calling zhttp_response_send.
        """
        return utils.lib.zhttp_request_recv(self._p, sock._p)

    def send(self, client, timeout, arg, arg2):
        """
        Send a request to zhttp_client.
        Url and the request path will be concatenated.
        This behavior is useful for url rewrite and reverse proxy.

        Send also allow two user provided arguments which will be returned with the response.
        The reason for two, is to be able to pass around the server connection when forwarding requests or both a callback function and an arg.
        """
        return utils.lib.zhttp_request_send(self._p, client._p, timeout, arg._p, arg2._p)

    def method(self):
        """
        Get the request method
        """
        return utils.lib.zhttp_request_method(self._p)

    def set_method(self, method):
        """
        Set the request method
        """
        utils.lib.zhttp_request_set_method(self._p, utils.to_bytes(method))

    def url(self):
        """
        Get the request url.
        When receiving a request from http server this is only the path part of the url.
        """
        return utils.lib.zhttp_request_url(self._p)

    def set_url(self, url):
        """
        Set the request url
        When sending a request to http client this should be full url.
        """
        utils.lib.zhttp_request_set_url(self._p, utils.to_bytes(url))

    def content_type(self):
        """
        Get the request content type
        """
        return utils.lib.zhttp_request_content_type(self._p)

    def set_content_type(self, content_type):
        """
        Set the request content type
        """
        utils.lib.zhttp_request_set_content_type(self._p, utils.to_bytes(content_type))

    def content_length(self):
        """
        Get the content length of the request
        """
        return utils.lib.zhttp_request_content_length(self._p)

    def headers(self):
        """
        Get the headers of the request
        """
        return utils.lib.zhttp_request_headers(self._p)

    def content(self):
        """
        Get the content of the request.
        """
        return utils.lib.zhttp_request_content(self._p)

    def get_content(self):
        """
        Get the content of the request.
        """
        return utils.lib.zhttp_request_get_content(self._p)

    def set_content(self, content):
        """
        Set the content of the request.
        Content must by dynamically allocated string.
        Takes ownership of the content.
        """
        utils.lib.zhttp_request_set_content(self._p, utils.to_bytes(content))

    def set_content_const(self, content):
        """
        Set the content of the request..
        The content is assumed to be constant-memory and will therefore not be copied or deallocated in any way.
        """
        utils.lib.zhttp_request_set_content_const(self._p, utils.to_bytes(content))

    def reset_content(self):
        """
        Set the content to NULL
        """
        utils.lib.zhttp_request_reset_content(self._p)

    def match(self, method, path, *path_args):
        """
        Match the path of the request.
        Support wildcards with '%s' symbol inside the match string.
        Matching wildcards until the next '/', '?' or '\0'.
        On successful match the variadic arguments will be filled with the matching strings.
        On successful match the method is modifying the url field and break it into substrings.
        If you need to use the url, do it before matching or take a copy.

        User must not free the variadic arguments as they are part of the url.

        To use the percent symbol, just double it, e.g "%%something".

        Example:
        if (zhttp_request_match (request, "POST", "/send/%s/%s", &name, &id))
        """
        return utils.lib.zhttp_request_match(self._p, utils.to_bytes(method), utils.to_bytes(path), *path_args)

    @staticmethod
    def test(verbose):
        """
        Self test of this class.
        """
        utils.lib.zhttp_request_test(verbose)

################################################################################
#  THIS FILE IS 100% GENERATED BY ZPROJECT; DO NOT EDIT EXCEPT EXPERIMENTALLY  #
#  Read the zproject/README.md for information about making permanent changes. #
################################################################################



################################################################################
#  THIS FILE IS 100% GENERATED BY ZPROJECT; DO NOT EDIT EXCEPT EXPERIMENTALLY  #
#  Read the zproject/README.md for information about making permanent changes. #
################################################################################
from . import utils
from . import destructors
libczmq_destructors = destructors.lib

class Zarmour(object):
    """
    armoured text encoding and decoding
    """

    def __init__(self):
        """
        Create a new zarmour
        """
        p = utils.lib.zarmour_new()
        if p == utils.ffi.NULL:
            raise MemoryError("Could not allocate person")

        # ffi.gc returns a copy of the cdata object which will have the
        # destructor called when the Python object is GC'd:
        # https://cffi.readthedocs.org/en/latest/using.html#ffi-interface
        self._p = utils.ffi.gc(p, libczmq_destructors.zarmour_destroy_py)

    def encode(self, data, size):
        """
        Encode a stream of bytes into an armoured string. Returns the armoured
        string, or NULL if there was insufficient memory available to allocate
        a new string.
        """
        return utils.lib.zarmour_encode(self._p, data, size)

    def decode(self, data):
        """
        Decode an armoured string into a chunk. The decoded output is
        null-terminated, so it may be treated as a string, if that's what
        it was prior to encoding.
        """
        return utils.lib.zarmour_decode(self._p, utils.to_bytes(data))

    def mode(self):
        """
        Get the mode property.
        """
        return utils.lib.zarmour_mode(self._p)

    def mode_str(self):
        """
        Get printable string for mode.
        """
        return utils.lib.zarmour_mode_str(self._p)

    def set_mode(self, mode):
        """
        Set the mode property.
        """
        utils.lib.zarmour_set_mode(self._p, mode)

    def pad(self):
        """
        Return true if padding is turned on.
        """
        return utils.lib.zarmour_pad(self._p)

    def set_pad(self, pad):
        """
        Turn padding on or off. Default is on.
        """
        utils.lib.zarmour_set_pad(self._p, pad)

    def pad_char(self):
        """
        Get the padding character.
        """
        return utils.lib.zarmour_pad_char(self._p)

    def set_pad_char(self, pad_char):
        """
        Set the padding character.
        """
        utils.lib.zarmour_set_pad_char(self._p, pad_char._p)

    def line_breaks(self):
        """
        Return if splitting output into lines is turned on. Default is off.
        """
        return utils.lib.zarmour_line_breaks(self._p)

    def set_line_breaks(self, line_breaks):
        """
        Turn splitting output into lines on or off.
        """
        utils.lib.zarmour_set_line_breaks(self._p, line_breaks)

    def line_length(self):
        """
        Get the line length used for splitting lines.
        """
        return utils.lib.zarmour_line_length(self._p)

    def set_line_length(self, line_length):
        """
        Set the line length used for splitting lines.
        """
        utils.lib.zarmour_set_line_length(self._p, line_length)

    def print_py(self):
        """
        Print properties of object
        """
        utils.lib.zarmour_print(self._p)

    @staticmethod
    def test(verbose):
        """
        Self test of this class.
        """
        utils.lib.zarmour_test(verbose)

################################################################################
#  THIS FILE IS 100% GENERATED BY ZPROJECT; DO NOT EDIT EXCEPT EXPERIMENTALLY  #
#  Read the zproject/README.md for information about making permanent changes. #
################################################################################



################################################################################
#  THIS FILE IS 100% GENERATED BY ZPROJECT; DO NOT EDIT EXCEPT EXPERIMENTALLY  #
#  Read the zproject/README.md for information about making permanent changes. #
################################################################################
from . import utils
from . import destructors
libczmq_destructors = destructors.lib

class Zdir(object):
    """
    work with file-system directories
    """

    def __init__(self, path, parent):
        """
        Create a new directory item that loads in the full tree of the specified
        path, optionally located under some parent path. If parent is "-", then
        loads only the top-level directory, and does not use parent as a path.
        """
        p = utils.lib.zdir_new(utils.to_bytes(path), utils.to_bytes(parent))
        if p == utils.ffi.NULL:
            raise MemoryError("Could not allocate person")

        # ffi.gc returns a copy of the cdata object which will have the
        # destructor called when the Python object is GC'd:
        # https://cffi.readthedocs.org/en/latest/using.html#ffi-interface
        self._p = utils.ffi.gc(p, libczmq_destructors.zdir_destroy_py)

    def path(self):
        """
        Return directory path
        """
        return utils.lib.zdir_path(self._p)

    def modified(self):
        """
        Return last modification time for directory.
        """
        return utils.lib.zdir_modified(self._p)

    def cursize(self):
        """
        Return total hierarchy size, in bytes of data contained in all files
        in the directory tree.
        """
        return utils.lib.zdir_cursize(self._p)

    def count(self):
        """
        Return directory count
        """
        return utils.lib.zdir_count(self._p)

    def list(self):
        """
        Returns a sorted list of zfile objects; Each entry in the list is a pointer
        to a zfile_t item already allocated in the zdir tree. Do not destroy the
        original zdir tree until you are done with this list.
        """
        return utils.lib.zdir_list(self._p)

    def remove(self, force):
        """
        Remove directory, optionally including all files that it contains, at
        all levels. If force is false, will only remove the directory if empty.
        If force is true, will remove all files and all subdirectories.
        """
        utils.lib.zdir_remove(self._p, force)

    @staticmethod
    def diff(older, newer, alias):
        """
        Calculate differences between two versions of a directory tree.
        Returns a list of zdir_patch_t patches. Either older or newer may
        be null, indicating the directory is empty/absent. If alias is set,
        generates virtual filename (minus path, plus alias).
        """
        return utils.lib.zdir_diff(older._p, newer._p, utils.to_bytes(alias))

    def resync(self, alias):
        """
        Return full contents of directory as a zdir_patch list.
        """
        return utils.lib.zdir_resync(self._p, utils.to_bytes(alias))

    def cache(self):
        """
        Load directory cache; returns a hash table containing the SHA-1 digests
        of every file in the tree. The cache is saved between runs in .cache.
        """
        return utils.lib.zdir_cache(self._p)

    def fprint(self, file, indent):
        """
        Print contents of directory to open stream
        """
        utils.lib.zdir_fprint(self._p, file, indent)

    def print_py(self, indent):
        """
        Print contents of directory to stdout
        """
        utils.lib.zdir_print(self._p, indent)

    @staticmethod
    def watch(pipe, unused):
        """
        Create a new zdir_watch actor instance:

            zactor_t *watch = zactor_new (zdir_watch, NULL);

        Destroy zdir_watch instance:

            zactor_destroy (&watch);

        Enable verbose logging of commands and activity:

            zstr_send (watch, "VERBOSE");

        Subscribe to changes to a directory path:

            zsock_send (watch, "ss", "SUBSCRIBE", "directory_path");

        Unsubscribe from changes to a directory path:

            zsock_send (watch, "ss", "UNSUBSCRIBE", "directory_path");

        Receive directory changes:
            zsock_recv (watch, "sp", &path, &patches);

            // Delete the received data.
            free (path);
            zlist_destroy (&patches);
        """
        utils.lib.zdir_watch(pipe._p, unused._p)

    @staticmethod
    def test(verbose):
        """
        Self test of this class.
        """
        utils.lib.zdir_test(verbose)

################################################################################
#  THIS FILE IS 100% GENERATED BY ZPROJECT; DO NOT EDIT EXCEPT EXPERIMENTALLY  #
#  Read the zproject/README.md for information about making permanent changes. #
################################################################################



################################################################################
#  THIS FILE IS 100% GENERATED BY ZPROJECT; DO NOT EDIT EXCEPT EXPERIMENTALLY  #
#  Read the zproject/README.md for information about making permanent changes. #
################################################################################
from . import utils
from . import destructors
libczmq_destructors = destructors.lib

class Zhashx(object):
    """
    extended generic type-free hash container
    """

    def __init__(self):
        """
        Create a new, empty hash container
        """
        p = utils.lib.zhashx_new()
        if p == utils.ffi.NULL:
            raise MemoryError("Could not allocate person")

        # ffi.gc returns a copy of the cdata object which will have the
        # destructor called when the Python object is GC'd:
        # https://cffi.readthedocs.org/en/latest/using.html#ffi-interface
        self._p = utils.ffi.gc(p, libczmq_destructors.zhashx_destroy_py)

    @staticmethod
    def unpack(frame):
        """
        Unpack binary frame into a new hash table. Packed data must follow format
        defined by zhashx_pack. Hash table is set to autofree. An empty frame
        unpacks to an empty hash table.
        """
        return utils.lib.zhashx_unpack(frame._p)

    @staticmethod
    def unpack_own(frame, deserializer):
        """
        Same as unpack but uses a user-defined deserializer function to convert
        a longstr back into item format.
        """
        return utils.lib.zhashx_unpack_own(frame._p, deserializer)

    def insert(self, key, item):
        """
        Insert item into hash table with specified key and item.
        If key is already present returns -1 and leaves existing item unchanged
        Returns 0 on success.
        """
        return utils.lib.zhashx_insert(self._p, key._p, item._p)

    def update(self, key, item):
        """
        Update or insert item into hash table with specified key and item. If the
        key is already present, destroys old item and inserts new one. If you set
        a container item destructor, this is called on the old value. If the key
        was not already present, inserts a new item. Sets the hash cursor to the
        new item.
        """
        utils.lib.zhashx_update(self._p, key._p, item._p)

    def delete(self, key):
        """
        Remove an item specified by key from the hash table. If there was no such
        item, this function does nothing.
        """
        utils.lib.zhashx_delete(self._p, key._p)

    def purge(self):
        """
        Delete all items from the hash table. If the key destructor is
        set, calls it on every key. If the item destructor is set, calls
        it on every item.
        """
        utils.lib.zhashx_purge(self._p)

    def lookup(self, key):
        """
        Return the item at the specified key, or null
        """
        return utils.lib.zhashx_lookup(self._p, key._p)

    def rename(self, old_key, new_key):
        """
        Reindexes an item from an old key to a new key. If there was no such
        item, does nothing. Returns 0 if successful, else -1.
        """
        return utils.lib.zhashx_rename(self._p, old_key._p, new_key._p)

    def freefn(self, key, free_fn):
        """
        Set a free function for the specified hash table item. When the item is
        destroyed, the free function, if any, is called on that item.
        Use this when hash items are dynamically allocated, to ensure that
        you don't have memory leaks. You can pass 'free' or NULL as a free_fn.
        Returns the item, or NULL if there is no such item.
        """
        return utils.lib.zhashx_freefn(self._p, key._p, free_fn)

    def size(self):
        """
        Return the number of keys/items in the hash table
        """
        return utils.lib.zhashx_size(self._p)

    def keys(self):
        """
        Return a zlistx_t containing the keys for the items in the
        table. Uses the key_duplicator to duplicate all keys and sets the
        key_destructor as destructor for the list.
        """
        return utils.lib.zhashx_keys(self._p)

    def values(self):
        """
        Return a zlistx_t containing the values for the items in the
        table. Uses the duplicator to duplicate all items and sets the
        destructor as destructor for the list.
        """
        return utils.lib.zhashx_values(self._p)

    def first(self):
        """
        Simple iterator; returns first item in hash table, in no given order,
        or NULL if the table is empty. This method is simpler to use than the
        foreach() method, which is deprecated. To access the key for this item
        use zhashx_cursor(). NOTE: do NOT modify the table while iterating.
        """
        return utils.lib.zhashx_first(self._p)

    def next(self):
        """
        Simple iterator; returns next item in hash table, in no given order,
        or NULL if the last item was already returned. Use this together with
        zhashx_first() to process all items in a hash table. If you need the
        items in sorted order, use zhashx_keys() and then zlistx_sort(). To
        access the key for this item use zhashx_cursor(). NOTE: do NOT modify
        the table while iterating.
        """
        return utils.lib.zhashx_next(self._p)

    def cursor(self):
        """
        After a successful first/next method, returns the key for the item that
        was returned. This is a constant string that you may not modify or
        deallocate, and which lasts as long as the item in the hash. After an
        unsuccessful first/next, returns NULL.
        """
        return utils.lib.zhashx_cursor(self._p)

    def comment(self, format, *format_args):
        """
        Add a comment to hash table before saving to disk. You can add as many
        comment lines as you like. These comment lines are discarded when loading
        the file. If you use a null format, all comments are deleted.
        """
        utils.lib.zhashx_comment(self._p, format, *format_args)

    def save(self, filename):
        """
        Save hash table to a text file in name=value format. Hash values must be
        printable strings; keys may not contain '=' character. Returns 0 if OK,
        else -1 if a file error occurred.
        """
        return utils.lib.zhashx_save(self._p, utils.to_bytes(filename))

    def load(self, filename):
        """
        Load hash table from a text file in name=value format; hash table must
        already exist. Hash values must printable strings; keys may not contain
        '=' character. Returns 0 if OK, else -1 if a file was not readable.
        """
        return utils.lib.zhashx_load(self._p, utils.to_bytes(filename))

    def refresh(self):
        """
        When a hash table was loaded from a file by zhashx_load, this method will
        reload the file if it has been modified since, and is "stable", i.e. not
        still changing. Returns 0 if OK, -1 if there was an error reloading the
        file.
        """
        return utils.lib.zhashx_refresh(self._p)

    def pack(self):
        """
        Serialize hash table to a binary frame that can be sent in a message.
        The packed format is compatible with the 'dictionary' type defined in
        http://rfc.zeromq.org/spec:35/FILEMQ, and implemented by zproto:

           ; A list of name/value pairs
           dictionary      = dict-count *( dict-name dict-value )
           dict-count      = number-4
           dict-value      = longstr
           dict-name       = string

           ; Strings are always length + text contents
           longstr         = number-4 *VCHAR
           string          = number-1 *VCHAR

           ; Numbers are unsigned integers in network byte order
           number-1        = 1OCTET
           number-4        = 4OCTET

        Comments are not included in the packed data. Item values MUST be
        strings.
        """
        return utils.lib.zhashx_pack(self._p)

    def pack_own(self, serializer):
        """
        Same as pack but uses a user-defined serializer function to convert items
        into longstr.
        """
        return utils.lib.zhashx_pack_own(self._p, serializer)

    def dup(self):
        """
        Make a copy of the list; items are duplicated if you set a duplicator
        for the list, otherwise not. Copying a null reference returns a null
        reference. Note that this method's behavior changed slightly for CZMQ
        v3.x, as it does not set nor respect autofree. It does however let you
        duplicate any hash table safely. The old behavior is in zhashx_dup_v2.
        """
        return utils.lib.zhashx_dup(self._p)

    def set_destructor(self, destructor):
        """
        Set a user-defined deallocator for hash items; by default items are not
        freed when the hash is destroyed.
        """
        utils.lib.zhashx_set_destructor(self._p, destructor)

    def set_duplicator(self, duplicator):
        """
        Set a user-defined duplicator for hash items; by default items are not
        copied when the hash is duplicated.
        """
        utils.lib.zhashx_set_duplicator(self._p, duplicator)

    def set_key_destructor(self, destructor):
        """
        Set a user-defined deallocator for keys; by default keys are freed
        when the hash is destroyed using free().
        """
        utils.lib.zhashx_set_key_destructor(self._p, destructor)

    def set_key_duplicator(self, duplicator):
        """
        Set a user-defined duplicator for keys; by default keys are duplicated
        using strdup.
        """
        utils.lib.zhashx_set_key_duplicator(self._p, duplicator)

    def set_key_comparator(self, comparator):
        """
        Set a user-defined comparator for keys; by default keys are
        compared using strcmp.
        The callback function should return zero (0) on matching
        items.
        """
        utils.lib.zhashx_set_key_comparator(self._p, comparator)

    def set_key_hasher(self, hasher):
        """
        Set a user-defined hash function for keys; by default keys are
        hashed by a modified Bernstein hashing function.
        """
        utils.lib.zhashx_set_key_hasher(self._p, hasher)

    def dup_v2(self):
        """
        Make copy of hash table; if supplied table is null, returns null.
        Does not copy items themselves. Rebuilds new table so may be slow on
        very large tables. NOTE: only works with item values that are strings
        since there's no other way to know how to duplicate the item value.
        """
        return utils.lib.zhashx_dup_v2(self._p)

    @staticmethod
    def test(verbose):
        """
        Self test of this class.
        """
        utils.lib.zhashx_test(verbose)

################################################################################
#  THIS FILE IS 100% GENERATED BY ZPROJECT; DO NOT EDIT EXCEPT EXPERIMENTALLY  #
#  Read the zproject/README.md for information about making permanent changes. #
################################################################################



from __future__ import print_function
import unittest
import sys, os, time
from ctypes import *
from czmq import *

# http://python3porting.com/problems.html#common-migration-problems
PY3 = sys.version_info > (3,)

if PY3:
    # Python 3 does not have a "cmp" builtin.
    def cmp(a, b):
        return (a > b) - (a < b)

# Part test, part example of how to use python bindings.
# Note the del statements are just to explicitly test functionality, it is generally not required.

class TestCZMQ(unittest.TestCase):
    verbose = int(os.environ.get('VERBOSE', '0'))

    def test_zdir(self):
        older = Zdir(b'.', None)
        self.assertTrue(older)
        if self.verbose:
            older.print(0)

        newer = Zdir(b'..', None)
        self.assertTrue(newer)

        patches = Zdir.diff(older, newer, b'/')
        self.assertTrue(patches)

        while patches.size():
            patch = ZdirPatch(patches.pop(), True)

        nosuch = Zdir(b"does-not-exist", None)
        self.assertFalse(nosuch)

        # zdir_watch test:
        watch = Zactor(zactor_fn(lib.zdir_watch), None)
        self.assertTrue(watch)

        if self.verbose:
            watch.sock().send(b"s", b"VERBOSE")
            self.assertEqual(watch.sock().wait(), 0)

        # need to create a file in the test directory we're watching
        # in order to ensure the directory exists
        initfile = Zfile(b"./zdir-test-dir", b"initial_file")
        self.assertTrue(initfile)
        initfile.output()
        if PY3:
            with initfile.handle() as hdl:  # Avoid ResourceWarning: unclosed file.
                hdl.write(b"initial file\n")
        else:
            initfile.handle().write(b"initial file\n")
        initfile.close()

        try:
            stable_age = float(file_stable_age_msec()) / 1000.0 + 0.050
        except Exception:
            stable_age = 5.050
        time.sleep(stable_age) # wait for initial file to become 'stable'

        watch.sock().send(b"si", b"TIMEOUT", 100)
        self.assertEqual(watch.sock().wait(), 0)

        watch.sock().send(b"ss", b"SUBSCRIBE", b"zdir-test-dir")
        self.assertEqual(watch.sock().wait(), 0)

        watch.sock().send(b"ss", b"UNSUBSCRIBE", b"zdir-test-dir")
        self.assertEqual(watch.sock().wait(), 0)

        watch.sock().send(b"ss", b"SUBSCRIBE", b"zdir-test-dir")
        self.assertEqual(watch.sock().wait(), 0)

        newfile = Zfile(b"zdir-test-dir", b"test_abc")
        newfile.output()
        if PY3:
            with newfile.handle() as hdl:  # Avoid ResourceWarning: unclosed file.
                hdl.write(b"test file\n")
        else:
            newfile.handle().write(b"test file\n")
        newfile.close()

        #watch_poll = Zpoller(watch, None)

        # poll for a certain timeout before giving up and failing the test.
        #assert(watch_poll.wait(1001) == watch)

        # wait for notification of the file being added
        path = c_char_p()
        patches = zlist_p()
        rc = watch.sock().recv(b"sp", byref(path), byref(patches))
        self.assertEqual(rc, 0)
        patches = Zlist(patches, True)

        self.assertEqual(string_at(path), b"zdir-test-dir")
        libc.free (path)

        self.assertEqual(patches.size(), 1)

        patch = ZdirPatch(patches.pop(), True)
        self.assertEqual(patch.path(), b"zdir-test-dir")

        patch_file = patch.file()
        self.assertEqual(patch_file.filename(b""), b"zdir-test-dir/test_abc")

        del patch
        del patches

        # remove the file
        newfile.remove()
        del newfile

        # poll for a certain timeout before giving up and failing the test.
        #assert(watch_poll.wait(1001) == watch)

        # wait for notification of the file being removed
        path = c_char_p()
        patches = zlist_p()
        rc = watch.sock().recv(b"sp", byref(path), byref(patches))
        self.assertEqual(rc, 0)
        patches = Zlist(patches, True)

        self.assertEqual(string_at(path), b"zdir-test-dir")
        libc.free (path)

        self.assertEqual(patches.size(), 1)

        patch = ZdirPatch(patches.pop(), True)
        self.assertEqual(patch.path(), b"zdir-test-dir")

        patch_file = patch.file()
        self.assertEqual(patch_file.filename(b""), b"zdir-test-dir/test_abc")

        del patch
        del patches

        #zpoller_destroy (&watch_poll)
        del watch

        # clean up by removing the test directory.
        testdir = Zdir(b"zdir-test-dir", None)
        testdir.remove(True)
        del testdir

    def test_zdir_patch(self):
        file = Zfile(b".", b"bilbo")
        self.assertTrue(file)
        patch = ZdirPatch(b".", file, ZdirPatch.CREATE, b"/")
        self.assertTrue(patch)
        del file

        file = patch.file()
        self.assertTrue(file)
        self.assertEqual(file.filename(b"."), b"bilbo")
        self.assertEqual(patch.vpath(), b"/bilbo")
        del patch

    def test_zfile(self):
        file = Zfile(None, b"bilbo")
        self.assertTrue(file)
        self.assertEqual(file.filename(b"."), b"bilbo")
        self.assertFalse(file.is_readable())
        del file

        # Create a test file in some random subdirectory
        file = Zfile(b"./this/is/a/test", b"bilbo")
        self.assertTrue(file)
        self.assertEqual(file.output(), 0)
        #chunk = Zchunk.new(None, 100)
        #self.assertTrue(chunk)
        #chunk.fill(0, 100)

        # Write 100 bytes at position 1,000,000 in the file
        #self.assertEqual(file.write(chunk, 1000000), 0)
        #del chunk
        file.close()
        self.assertTrue(file.is_readable())
        #self.assertEqual(file.cursize(), 1000100)
        #self.assertFalse(file.is_stable())
        self.assertTrue(file.digest())

        # Now truncate file from outside
        f = open('./this/is/a/test/bilbo', 'wb')
        f.truncate()
        f.write(b"Hello, World\n")
        f.close()
        del f

        try:
            stable_age = float(file_stable_age_msec()) / 1000.0 + 0.050
        except Exception:
            stable_age = 5.050

        self.assertTrue(file.has_changed())
        self.assertFalse(file.is_stable())
        time.sleep(stable_age) # just over a threshold value passes...
        self.assertTrue(file.has_changed())
        self.assertFalse(file.is_stable())

        file.restat() # restat.
        self.assertTrue(file.is_stable())
        self.assertFalse(file.has_changed())

        self.assertEqual(file.digest(), b"4AB299C8AD6ED14F31923DD94F8B5F5CB89DFB54")

        # Check we can read from file
        self.assertEqual(file.input(), 0)
        #chunk = file.read(1000100, 0)
        #self.assertTrue(chunk)
        #self.assertEqual(chunk.size(), 13)
        #del chunk
        file.close()

        # Check we can read lines from file
        self.assertEqual(file.input(), 0)
        line = file.readln()
        self.assertEqual(line, b"Hello, World")
        line = file.readln()
        self.assertIsNone(line)
        file.close()

        # Try some fun with symbolic links
        link = Zfile(b"./this/is/a/test", b"bilbo.ln")
        self.assertTrue(link)
        self.assertEqual(link.output(), 0)
        if PY3:
            with link.handle() as hdl:  # Avoid ResourceWarning: unclosed file.
                hdl.write(b"./this/is/a/test/bilbo\n")
        else:
            link.handle().write(b"./this/is/a/test/bilbo\n")

        del link

        link = Zfile(b"./this/is/a/test", b"bilbo.ln")
        self.assertTrue(link)
        self.assertEqual(link.input(), 0)
        #chunk = link.read(1000100, 0)
        #self.assertTrue(chunk)
        #self.assertEqual(chunk.size(), 13)
        #del chunk
        del link

        # Remove file and directory
        dir = Zdir(b"./this", None)
        self.assertTrue(dir)
        self.assertEqual(dir.cursize(), 26)
        dir.remove(True)
        self.assertEqual(dir.cursize(), 0)
        del dir

        # Check we can no longer read from file
        self.assertTrue(file.is_readable())
        file.restat()
        self.assertFalse(file.is_readable())
        self.assertEqual(file.input(), -1)
        del file

    def test_zframe(self):
        # Create two PAIR sockets and connect over inproc
        output = Zsock.new_pair (b"@inproc://zframe.test")
        self.assertTrue(output)
        input = Zsock.new_pair (b">inproc://zframe.test")
        self.assertTrue(input)

        # Send five different frames, test Zframe.MORE
        for frame_nbr in range(5):
            frame = Zframe(b"Hello", 5)
            self.assertTrue(frame)
            self.assertEqual(Zframe.send(frame, output, Zframe.MORE), 0)

        # Send same frame five times, test ZFRAME_REUSE
        frame = Zframe(b"Hello", 5)
        self.assertTrue(frame)
        for frame_nbr in range(5):
            self.assertEqual(Zframe.send(frame, output, Zframe.MORE + Zframe.REUSE), 0)
        self.assertTrue(frame)
        copy = frame.dup()
        self.assertTrue(frame.eq(copy))
        self.assertEqual(copy.size(), 5)
        del copy
        self.assertFalse(frame.eq(None))

        # Test zframe_new_empty
        frame = Zframe.new_empty()
        self.assertTrue(frame)
        self.assertEqual(frame.size(), 0)
        del frame

        # Send END frame
        frame = Zframe(b"NOT", 3)
        self.assertTrue(frame)
        frame.reset(b"END", 3)
        string = frame.strhex()
        self.assertEqual(string, b"454E44")
        string = frame.strdup()
        self.assertEqual(string, b"END")
        self.assertEqual(Zframe.send(frame, output, 0), 0)

        # Read and count until we receive END
        frame_nbr = 0
        while True:
            frame = Zframe.recv(input)
            if frame.streq(b"END"):
                break

            self.assertTrue(frame.more())
            frame.set_more(0)
            self.assertEqual(frame.more(), 0)
            frame_nbr += 1
        self.assertEqual(frame_nbr, 10)

    def test_zhash(self):
        hash = Zhash()
        self.assertTrue(hash)
        self.assertEqual(hash.size(), 0)
        self.assertFalse(hash.first())
        self.assertFalse(hash.cursor())

        # Insert some items
        # Note the bindings for zhash are pretty pointless for actually storing
        # things - we have to keep our ctypes buffer objects around to keep the
        # memory alive
        buffer1 = create_string_buffer(b"dead beef")
        self.assertEqual(hash.insert(b"DEADBEEF", buffer1), 0)
        self.assertEqual(string_at(hash.first()), b"dead beef")
        self.assertEqual(hash.cursor(), b"DEADBEEF")

        buffer2 = create_string_buffer(b"a bad cafe")
        self.assertEqual(hash.insert(b"ABADCAFE", buffer2), 0)
        buffer3 = create_string_buffer(b"coded bad")
        self.assertEqual(hash.insert(b"C0DEDBAD", buffer3), 0)
        buffer4 = create_string_buffer(b"dead food")
        self.assertEqual(hash.insert(b"DEADF00D", b"dead food"), 0)
        self.assertEqual(hash.size(), 4)

        # Look for existing items
        self.assertEqual(string_at(hash.lookup(b"DEADBEEF")), b"dead beef")
        self.assertEqual(string_at(hash.lookup(b"ABADCAFE")), b"a bad cafe")
        self.assertEqual(string_at(hash.lookup(b"DEADF00D")), b"dead food")

        # Look for non-existent items
        self.assertFalse(hash.lookup(b"foo"))

        # Try to insert duplicate items
        self.assertEqual(hash.insert(b"DEADBEEF", b"foo"), -1)
        self.assertEqual(string_at(hash.lookup(b"DEADBEEF")), b"dead beef")

        # Some rename tests

        # Valid rename, key is now LIVEBEEF
        self.assertEqual(hash.rename(b"DEADBEEF", b"LIVEBEEF"), 0)
        self.assertEqual(string_at(hash.lookup(b"LIVEBEEF")), b"dead beef")

        # Trying to rename an unknown item to a non-existent key
        self.assertEqual(hash.rename(b"WHATBEEF", b"NONESUCH"), -1)

        # Trying to rename an unknown item to an existing key
        self.assertEqual(hash.rename(b"WHATBEEF", b"LIVEBEEF"), -1)
        self.assertEqual(string_at(hash.lookup(b"LIVEBEEF")), b"dead beef")

        # Trying to rename an existing item to another existing item
        self.assertEqual(hash.rename(b"LIVEBEEF", b"ABADCAFE"), -1)
        self.assertEqual(string_at(hash.lookup(b"LIVEBEEF")), b"dead beef")
        self.assertEqual(string_at(hash.lookup(b"ABADCAFE")), b"a bad cafe")

        # Test keys method
        keys = hash.keys()
        self.assertEqual(keys.size(), 4)
        del keys

        # Test dup method
        copy = hash.dup()
        self.assertEqual(copy.size(), 4)
        self.assertEqual(string_at(copy.lookup(b"LIVEBEEF")), b"dead beef")
        del copy

        # Test pack/unpack methods
        frame = hash.pack()
        copy = Zhash.unpack(frame)
        del frame
        self.assertEqual(copy.size(), 4)
        self.assertEqual(string_at(copy.lookup(b"LIVEBEEF")), b"dead beef")
        del copy

        # Test save and load
        hash.comment(b"This is a test file")
        hash.comment(b"Created by %s", b"czmq_selftest")
        hash.save(b".cache")
        copy = Zhash()
        self.assertTrue(copy)
        copy.load(b".cache")
        self.assertEqual(string_at(copy.lookup(b"LIVEBEEF")), b"dead beef")
        del copy
        os.remove(b".cache")

        # Delete a item
        hash.delete(b"LIVEBEEF")
        self.assertFalse(hash.lookup(b"LIVEBEEF"))
        self.assertEqual(hash.size(), 3)

        # Test autofree automatically copies and frees string values
        # This means the python string doesn't need to persist, like above
        hash = Zhash()
        self.assertTrue(hash)
        hash.autofree()
        self.assertEqual(hash.insert(b"key1", b"This is a string"), 0)
        self.assertEqual(hash.insert(b"key2", b"Ring a ding ding"), 0)
        self.assertEqual(string_at(hash.lookup(b"key1")), b"This is a string")
        self.assertEqual(string_at(hash.lookup(b"key2")), b"Ring a ding ding")
        del hash

    def test_zlist(self):
        def _zlist_free(data):
            x = Zlist(c_void_p(data), True)
            del x
        zlist_free = zlist_free_fn(_zlist_free)

        def _compare(item1, item2):
            return cmp(string_at(item1), string_at(item2)) > 0
        zlist_compare = zlist_compare_fn(_compare)

        list = Zlist()
        self.assertTrue(list)
        self.assertEqual(list.size(), 0)

        # Three items we'll use as test data
        # List items are void *, not particularly strings
        cheese = create_string_buffer(b"boursin")
        bread = create_string_buffer(b"baguette")
        wine = create_string_buffer(b"bordeaux")

        list.append(cheese)
        self.assertEqual(list.size(), 1)
        list.append(bread)
        self.assertEqual(list.size(), 2)
        list.append(wine)
        self.assertEqual(list.size(), 3)

        self.assertEqual(list.head().value, addressof(cheese))
        self.assertEqual(list.next().value, addressof(cheese))

        self.assertEqual(list.first().value, addressof(cheese))
        self.assertEqual(list.tail().value, addressof(wine))
        self.assertEqual(list.next().value, addressof(bread))

        self.assertEqual(list.first().value, addressof(cheese))
        self.assertEqual(list.next().value, addressof(bread))
        self.assertEqual(list.next().value, addressof(wine))
        self.assertFalse(list.next())
        # After we reach end of list, next wraps around
        self.assertEqual(list.next().value, addressof(cheese))
        self.assertEqual(list.size(), 3)

        list.remove(wine)
        self.assertEqual(list.size(), 2)

        self.assertEqual(list.first().value, addressof(cheese))
        list.remove(cheese)
        self.assertEqual(list.size(), 1)
        self.assertEqual(list.first().value, addressof(bread))

        list.remove(bread)
        self.assertEqual(list.size(), 0)

        list.append(cheese)
        list.append(bread)
        self.assertEqual(list.last().value, addressof(bread))
        list.remove(bread)
        self.assertEqual(list.last().value, addressof(cheese))
        list.remove(cheese)
        self.assertFalse(list.last())

        list.push(cheese)
        self.assertEqual(list.size(), 1)
        self.assertEqual(list.first().value, addressof(cheese))

        list.push(bread)
        self.assertEqual(list.size(), 2)
        self.assertEqual(list.first().value, addressof(bread))
        self.assertEqual(list.item().value, addressof(bread))

        list.append(wine)
        self.assertEqual(list.size(), 3)
        self.assertEqual(list.first().value, addressof(bread))

        sub_list = list.dup()
        self.assertTrue(sub_list)
        self.assertEqual(sub_list.size(), 3)

        list.sort(zlist_compare)
        self.assertEqual(list.pop().value, addressof(bread))
        self.assertEqual(list.pop().value, addressof(wine))
        self.assertEqual(list.pop().value, addressof(cheese))
        self.assertEqual(list.size(), 0)

        self.assertEqual(sub_list.size(), 3)
        list.push(sub_list)
        sub_list_2 = sub_list.dup()
        list.append(sub_list_2)

        ret = list.freefn(sub_list, zlist_free, False)
        sub_list.allow_destruct = False # Can't double-free
        self.assertEqual(ret.value, cast(sub_list._as_parameter_, c_void_p).value)

        ret = list.freefn(sub_list_2, zlist_free, True)
        sub_list_2.allow_destruct = False # Can't double-free
        self.assertEqual(ret.value, cast(sub_list_2._as_parameter_, c_void_p).value)
        del list

        # Test autofree functionality
        list = Zlist()
        self.assertTrue(list)
        list.autofree()
        list.push(b"bread")
        list.append(b"cheese")
        self.assertEqual(list.size(), 2)
        self.assertEqual(string_at(list.first()), b"bread")
        item = list.pop()
        self.assertEqual(string_at(item), b"bread")
        libc.free(item)
        item = list.pop()
        self.assertEqual(string_at(item), b"cheese")
        libc.free(item)

    def test_zloop(self):
        def _cancel_timer_event(loop, timer_id, arg):
            # We are handling timer 2, and will cancel timer 1
            self.assertIsInstance(arg, int)
            return Zloop(loop, False).timer_end(arg)
        cancel_timer_event = zloop_timer_fn(_cancel_timer_event)

        def _timer_event(loop, timer_id, arg):
            output.send(b's', b"PING")
            return 0
        timer_event = zloop_timer_fn(_timer_event)

        def _socket_event(loop, handle, arg):
            # Just end the reactor
            return -1
        socket_event = zloop_reader_fn(_socket_event)

        def _timer_event3(loop, timer_id, called):
            c_bool_p(called).contents = True
            # end the reactor
            return -1
        timer_event3 = zloop_timer_fn(_timer_event3)

        # Create two PAIR sockets and connect over inproc
        output = Zsock.new_pair(b"@inproc://zloop.test")
        self.assertTrue(output)
        input = Zsock.new_pair(b">inproc://zloop.test")
        self.assertTrue(input)

        loop = Zloop()
        self.assertTrue(loop)
        loop.set_verbose(self.verbose)

        # Create a timer that will be cancelled
        timer_id = loop.timer(1000, 1, timer_event, None)
        loop.timer(5, 1, cancel_timer_event, timer_id)

        # After 20 msecs, send a ping message to output3
        loop.timer(20, 1, timer_event, output)

        # Set up some tickets that will never expire
        loop.set_ticket_delay(10000)
        ticket1 = loop.ticket(timer_event, None)
        ticket2 = loop.ticket(timer_event, None)
        ticket3 = loop.ticket(timer_event, None)

        # When we get the ping message, end the reactor
        self.assertEqual(loop.reader(input, socket_event, None), 0)
        loop.reader_set_tolerant(input)
        loop.start()

        loop.ticket_delete(ticket1)
        loop.ticket_delete(ticket2)
        loop.ticket_delete(ticket3)
        del loop

        # Check whether loop properly ignores zsys_interrupted flag
        # when asked to
        #loop = Zloop()

        #timer_event_called = c_bool(False)
        #loop.timer(1, 1, timer_event3, byref(timer_event_called))

        #zsys_interrupted = 1
        #zloop_start (loop)
        ## zloop returns immediately without giving any handler a chance to run
        #self.assertFalse(timer_event_called)

        #zloop_ignore_interrupts (loop)
        #zloop_start (loop)
        # zloop runs the handler which will terminate the loop
        #self.assertTrue(timer_event_called)
        #zsys_interrupted = 0

        # cleanup
        #del loop
        #self.assertFalse(loop)

    def test_zmsg(self):
        # Create two PAIR sockets and connect over inproc
        output = Zsock.new_pair(b"@inproc://zmsg.test")
        self.assertTrue(output)
        input_ = Zsock.new_pair(b">inproc://zmsg.test")
        self.assertTrue(input_)

        # Test send and receive of single-frame message
        msg = Zmsg()
        self.assertTrue(msg)
        frame = Zframe(b"Hello", 5)
        self.assertTrue(frame)
        msg.prepend(frame)
        self.assertEqual(msg.size(), 1)
        self.assertEqual(msg.content_size(), 5)
        self.assertEqual(Zmsg.send(msg, output), 0)
        self.assertFalse(msg)

        msg = Zmsg.recv(input_)
        self.assertTrue(msg)
        self.assertEqual(msg.size(), 1)
        self.assertEqual(msg.content_size(), 5)
        del msg

        # Test send and receive of multi-frame message
        msg = Zmsg()
        self.assertTrue(msg)
        self.assertEqual(msg.addmem(b"Frame0", 6), 0)
        self.assertEqual(msg.addmem(b"Frame1", 6), 0)
        self.assertEqual(msg.addmem(b"Frame2", 6), 0)
        self.assertEqual(msg.addmem(b"Frame3", 6), 0)
        self.assertEqual(msg.addmem(b"Frame4", 6), 0)
        self.assertEqual(msg.addmem(b"Frame5", 6), 0)
        self.assertEqual(msg.addmem(b"Frame6", 6), 0)
        self.assertEqual(msg.addmem(b"Frame7", 6), 0)
        self.assertEqual(msg.addmem(b"Frame8", 6), 0)
        self.assertEqual(msg.addmem(b"Frame9", 6), 0)
        copy = msg.dup()
        self.assertTrue(copy)
        self.assertEqual(Zmsg.send(copy, output), 0)
        self.assertEqual(Zmsg.send(msg, output), 0)

        copy = Zmsg.recv(input_)
        self.assertTrue(copy)
        self.assertEqual(copy.size(), 10)
        self.assertEqual(copy.content_size(), 60)
        del copy

        msg = Zmsg.recv(input_)
        self.assertTrue(msg)
        self.assertEqual(msg.size(), 10)
        self.assertEqual(msg.content_size(), 60)

        # Save to a file, read back
        f = open("zmsg.test", "w")
        self.assertEqual(msg.save(f), 0)
        f.close()

        f = open ("zmsg.test", "r")
        self.assertEqual(msg.save(f), -1)
        f.close()
        del msg

        f = open ("zmsg.test", "r")
        msg = Zmsg.load(f)
        self.assertTrue(msg)
        f.close()
        os.remove ("zmsg.test")
        self.assertEqual(msg.size(), 10)
        self.assertEqual(msg.content_size(), 60)

        # Remove all frames except first and last
        for frame_nbr in range(8):
            msg.first()
            frame = msg.next()
            msg.remove(frame)

        # Test message frame manipulation
        self.assertEqual(msg.size(), 2)
        frame = msg.last()
        self.assertTrue(frame.streq(b"Frame9"))
        self.assertEqual(msg.content_size(), 12)
        frame = Zframe(b"Address", 7)
        self.assertTrue(frame)
        msg.prepend(frame)
        self.assertEqual(msg.size(), 3)
        self.assertEqual(msg.addstr(b"Body"), 0)
        self.assertEqual(msg.size(), 4)
        frame = msg.pop()
        del frame
        self.assertEqual(msg.size(), 3)
        body = msg.popstr()
        self.assertEqual(body, b"Frame0")
        del msg

        # Test encoding/decoding
        msg = Zmsg()
        self.assertTrue(msg)
        blank = create_string_buffer(100000)
        self.assertEqual(msg.addmem(blank, 0), 0)
        self.assertEqual(msg.addmem(blank, 1), 0)
        self.assertEqual(msg.addmem(blank, 253), 0)
        self.assertEqual(msg.addmem(blank, 254), 0)
        self.assertEqual(msg.addmem(blank, 255), 0)
        self.assertEqual(msg.addmem(blank, 256), 0)
        self.assertEqual(msg.addmem(blank, 65535), 0)
        self.assertEqual(msg.addmem(blank, 65536), 0)
        self.assertEqual(msg.addmem(blank, 65537), 0)
        del blank
        self.assertEqual(msg.size(), 9)
        frame = msg.encode()
        del msg
        msg = Zmsg.decode(frame)
        self.assertTrue(msg)
        del msg

        # Test submessages
        msg = Zmsg()
        self.assertTrue(msg)
        submsg = Zmsg()
        msg.pushstr(b"matr")
        submsg.pushstr(b"joska")
        self.assertEqual(msg.addmsg(submsg), 0)
        self.assertFalse(submsg)
        submsg = msg.popmsg()
        self.assertFalse(submsg)   # string "matr" is not encoded zmsg_t, so was discarded
        submsg = msg.popmsg()
        self.assertTrue(submsg)
        body = submsg.popstr()
        self.assertEqual(body, b"joska")
        del submsg
        frame = msg.pop()
        self.assertFalse(frame)
        del msg

        # Test comparison of two messages
        msg = Zmsg()
        msg.addstr(b"One")
        msg.addstr(b"Two")
        msg.addstr(b"Three")
        msg_other = Zmsg()
        msg_other.addstr(b"One")
        msg_other.addstr(b"Two")
        msg_other.addstr(b"One-Hundred")
        msg_dup = msg.dup()
        empty_msg = Zmsg()
        empty_msg_2 = Zmsg()
        self.assertTrue(msg.eq(msg_dup))
        self.assertFalse(msg.eq(msg_other))
        self.assertTrue(empty_msg.eq(empty_msg_2))
        self.assertFalse(msg.eq(None))
        del msg
        del msg_other
        del msg_dup
        del empty_msg

        # Test signal messages
        msg = Zmsg.new_signal(0)
        self.assertEqual(msg.signal(), 0)
        del msg
        msg = Zmsg.new_signal(-1)
        self.assertEqual(msg.signal(), 255)
        del msg

        # Now try methods on an empty message
        msg = Zmsg()
        self.assertTrue(msg)
        self.assertEqual(msg.size(), 0)
        self.assertFalse(msg.first())
        self.assertFalse(msg.last())
        self.assertFalse(msg.next())
        self.assertFalse(msg.pop())
        # Sending an empty message is valid and destroys the message
        self.assertEqual(Zmsg.send(msg, output), 0)
        self.assertFalse(msg)

    def test_zsock(self):
        writer = Zsock.new_push (b"@tcp://127.0.0.1:5560")
        self.assertTrue(writer)
        self.assertTrue(Zsock.resolve(writer) != writer)
        self.assertEqual(writer.type_str(), b"PUSH")

        # Check unbind
        self.assertEqual(writer.unbind(b"tcp://127.0.0.1:%d", 5560), 0)

        # In some cases and especially when running under Valgrind, doing
        # a bind immediately after an unbind causes an EADDRINUSE error.
        # Even a short sleep allows the OS to release the port for reuse.
        time.sleep(0.1)

        # Bind again
        self.assertEqual(writer.bind(b"tcp://127.0.0.1:%d", 5560), 5560)
        self.assertEqual(writer.endpoint(), b"tcp://127.0.0.1:5560")
#endif

        reader = Zsock.new_pull (b">tcp://127.0.0.1:5560")
        self.assertTrue(reader)
        self.assertTrue(Zsock.resolve(reader) != reader)
        self.assertEqual(reader.type_str(), b"PULL")

        # Basic Hello, World
        writer.send(b's', b"Hello, World")
        msg = Zmsg.recv(reader)
        self.assertTrue(msg)
        string = msg.popstr()
        self.assertEqual(string, b"Hello, World")
        del msg

        # Test resolve FD
        #fd = Zsock.fd(reader)
        #self.assertEqual(zsock_resolve (fd), None)

        # Test binding to ephemeral ports, sequential and random
        DYNAMIC_FIRST = 0xc000
        DYNAMIC_LAST = 0xffff
        port = writer.bind(b"tcp://127.0.0.1:*")
        self.assertTrue(port >= DYNAMIC_FIRST and port <= DYNAMIC_LAST)
        port = writer.bind(b"tcp://127.0.0.1:*[50000-]")
        self.assertTrue(port >= 50000 and port <= DYNAMIC_LAST)
        port = writer.bind(b"tcp://127.0.0.1:*[-50001]")
        self.assertTrue(port >= DYNAMIC_FIRST and port <= 50001)
        port = writer.bind(b"tcp://127.0.0.1:*[60000-60050]")
        self.assertTrue(port >= 60000 and port <= 60050)

        port = writer.bind(b"tcp://127.0.0.1:!")
        self.assertTrue(port >= DYNAMIC_FIRST and port <= DYNAMIC_LAST)
        port = writer.bind(b"tcp://127.0.0.1:![50000-]")
        self.assertTrue(port >= 50000 and port <= DYNAMIC_LAST)
        port = writer.bind(b"tcp://127.0.0.1:![-50001]")
        self.assertTrue(port >= DYNAMIC_FIRST and port <= 50001)
        port = writer.bind(b"tcp://127.0.0.1:![60000-60050]")
        self.assertTrue(port >= 60000 and port <= 60050)

        # Test zsock_attach method
        server = Zsock.new_dealer(b"")
        self.assertTrue(server)
        self.assertEqual(server.attach(b"@inproc://myendpoint,tcp://127.0.0.1:5556,inproc://others", True), 0)
        self.assertEqual(server.attach(b"", False), 0)
        self.assertEqual(server.attach(None, True), 0)
        self.assertEqual(server.attach(b">a,@b, c,, ", False), -1)
        del server

        # Test zsock_endpoint method
        self.assertEqual(writer.bind(b"inproc://test.%s", b"writer"), 0)
        self.assertEqual(writer.endpoint(), b"inproc://test.writer")

        # Test error state when connecting to an invalid socket type
        # ('txp://' instead of 'tcp://', typo intentional)
        self.assertEqual(reader.connect(b"txp://127.0.0.1:5560"), -1)

        # Test signal/wait methods
        self.assertEqual(writer.signal(123), 0)
        self.assertEqual(reader.wait(), 123)

        # Test zsock_send/recv pictures
        number1 = 123
        number2 = 123 * 123
        number4 = 123 * 123 * 123
        number8 = 123 * 123 * 123 * 123

        #chunk = zchunk_new (b"HELLO", 5)
        #self.assertTrue(chunk)
        frame = Zframe(b"WORLD", 5)
        self.assertTrue(frame)
        #hash = zhashx_new ()
        #self.assertTrue(hash)
        #hash.autofree()
        #hash.insert(b"1", b"value A")
        #hash.insert(b"2", b"value B")
        original = create_string_buffer(b"pointer")

        # Test zsock_recv into each supported type
        #writer.send(b"i1248zsbcfhp",
        #            -12345, number1, number2, number4, number8,
        #            b"This is a string", b"ABCDE", 5, chunk, frame, hash, original)
        writer.send(b"i1248zsbfp",
                    -12345, number1, number2, number4, c_longlong(number8),
                    b"This is a string", b"ABCDE", 5, frame, original)
        del frame
        #del chunk
        #del hash

        integer = c_int()
        number1 = c_byte()
        number2 = c_short()
        number4 = c_int()
        number8 = c_longlong()
        string = c_char_p()
        data = POINTER(c_ubyte)()
        size = c_size_t()
        #chunk = zchunk_p()
        frame = Zframe(c_void_p(), True)
        #hash = POINTER(zhashx_p)
        pointer = c_void_p()
        #rc = reader.recv(b"i1248zsbcfhp",
        #                 byref(integer), byref(number1), byref(number2), byref(number4), byref(number8),
        #                 byref(string), byref(data), byref(size), byref(chunk), byref(frame), byref(hash), byref(pointer))
        rc = reader.recv(b"i1248zsbfp",
                         byref(integer), byref(number1), byref(number2), byref(number4), byref(number8),
                         byref(string), byref(data), byref(size), byref(frame._as_parameter_), byref(pointer))
        self.assertEqual(rc, 0)
        self.assertEqual(integer.value, -12345)
        self.assertEqual(number1.value, 123)
        self.assertEqual(number2.value, 123 * 123)
        self.assertEqual(number4.value, 123 * 123 * 123)
        self.assertEqual(number8.value, 123 * 123 * 123 * 123)
        self.assertEqual(string_at(string), b"This is a string")
        self.assertEqual(string_at(data, size.value), b"ABCDE")
        self.assertEqual(size.value, 5)
        #self.assertEqual(memcmp (chunk.data(), b"HELLO", 5), 0)
        #self.assertEqual(chunk.size(), 5)
        self.assertTrue(frame.streq(b"WORLD"))
        self.assertEqual(frame.size(), 5)
        #value = string_at(hash.lookup(b"1"))
        #self.assertEqual(value, b"value A")
        #value = string_at(hash.lookup(b"2"))
        #self.assertEqual(value, b"value B")
        self.assertEqual(addressof(original), pointer.value)
        libc.free (string)
        libc.free (data)
        del frame
        #del chunk
        #del hash

        # Test zsock_recv of short message this lets us return a failure
        # with a status code and then nothing else the receiver will get
        # the status code and None/zero for all other values
        writer.send(b"i", -1)
        #reader.recv(b"izsbcfp",
        #    byref(integer), byref(string), byref(data), byref(size), byref(chunk), byref(frame), byref(pointer))
        frame = Zframe(c_void_p(), True)
        reader.recv(b"izsbfp",
            byref(integer), byref(string), byref(data), byref(size), byref(frame._as_parameter_), byref(pointer))
        self.assertEqual(integer.value, -1)
        self.assertEqual(string.value, None)
        self.assertFalse(data)
        self.assertEqual(size.value, 0)
        #self.assertEqual(chunk, None)
        self.assertFalse(frame)
        self.assertEqual(pointer.value, None)

        msg = Zmsg()
        msg.addstr(b"frame 1")
        msg.addstr(b"frame 2")
        writer.send(b"szm", b"header", msg)
        del msg

        msg = Zmsg(c_void_p(), True)
        reader.recv(b"szm", byref(string), byref(msg._as_parameter_))

        self.assertEqual(string_at(string), b"header")
        self.assertEqual(msg.size(), 2)
        self.assertEqual(msg.popstr(), b"frame 1")
        self.assertEqual(msg.popstr(), b"frame 2")
        #zstr_free (byref(string))
        libc.free(string)
        del msg

        # Test zsock_recv with null arguments
        #chunk = zchunk_new (b"HELLO", 5)
        #self.assertTrue(chunk)
        frame = Zframe(b"WORLD", 5)
        self.assertTrue(frame)
        #writer.send(b"izsbcfp",
        #            -12345, b"This is a string", b"ABCDE", 5, chunk, frame, original)
        writer.send(b"izsbfp",
                    -12345, b"This is a string", b"ABCDE", 5, frame, original)
        del frame
        #del chunk
        #reader.recv(b"izsbcfp", byref(integer), None, None, None, byref(chunk), None, None)
        reader.recv(b"izsbcfp", byref(integer), None, None, None, None, None, None)
        self.assertEqual(integer.value, -12345)
        #self.assertEqual(chunk.data(), b"HELLO", 5), 0)
        #self.assertEqual(chunk.size(), 5)
        #del chunk

        # Test zsock_bsend/brecv pictures with binary encoding
        frame = Zframe(b"Hello", 5)
        #chunk = Zchunk(b"World", 5)

        msg = Zmsg()
        msg.addstr(b"Hello")
        msg.addstr(b"World")

        #writer.bsend(b"1248sSpcfm",
        #             number1, number2, number4, number8,
        #             b"Hello, World",
        #             b"Goodbye cruel World!",
        #             original,
        #             chunk, frame, msg)
        writer.bsend(b"1248sSpfm",
                     number1, number2, number4, number8,
                     b"Hello, World",
                     b"Goodbye cruel World!",
                     original,
                     frame, msg)
        #del chunk
        del frame
        del msg

        frame = Zframe(c_void_p(), True)
        msg = Zmsg(c_void_p(), True)
        longstr = c_char_p()
        #reader.brecv(b"1248sSpcfm",
        #             byref(number1), byref(number2), byref(number4), byref(number8),
        #             byref(string), byref(longstr),
        #             byref(pointer),
        #             byref(chunk), byref(frame._as_parameter_), byref(msg._as_parameter_))
        reader.brecv(b"1248sSpfm",
                     byref(number1), byref(number2), byref(number4), byref(number8),
                     byref(string), byref(longstr),
                     byref(pointer),
                     byref(frame._as_parameter_), byref(msg._as_parameter_))
        self.assertEqual(number1.value, 123)
        self.assertEqual(number2.value, 123 * 123)
        self.assertEqual(number4.value, 123 * 123 * 123)
        self.assertEqual(number8.value, 123 * 123 * 123 * 123)
        self.assertEqual(string_at(string), b"Hello, World")
        self.assertEqual(string_at(longstr), b"Goodbye cruel World!")
        self.assertEqual(pointer.value, addressof(original))
        libc.free(longstr)
        #zstr_free (byref(longstr))
        #del chunk
        del frame
        del msg

        # Check that we can send a zproto format message
        #writer.bsend(b"1111sS4", 0xAA, 0xA0, 0x02, 0x01, b"key", b"value", 1234)
        #gossip = zgossip_msg_new ()
        #gossip.recv(reader)
        #self.assertEqual(gossip.id(), ZGOSSIP_MSG_PUBLISH)
        #del gossip

    def test_zactor(self):
        def _echo_actor(pipe, arg):
            # Do some initialization
            pipe = Zsock(pipe, False) # We don't own the pipe, so False.
            arg = string_at(arg)
            self.assertEqual(arg, b"Hello, World")
            pipe.signal(0) # startup ok

            terminated = False
            while not terminated:
                msg = Zmsg.recv(pipe)
                if not msg:
                    break # Interrupted

                command = msg.popstr()
                if command == b"$TERM":
                    # All actors must handle $TERM in this way
                    terminated = True
                elif command == b"ECHO":
                    # This is an example command for our test actor
                    Zmsg.send(msg, pipe)
                else:
                    self.fail(b"Unexpected message " + command)
        echo_actor = zactor_fn(_echo_actor) # ctypes function reference must live as long as the actor.

        arg_buffer = create_string_buffer(b"Hello, World") # argument buffer must live as long as the actor.
        actor = Zactor(echo_actor, arg_buffer)
        self.assertTrue(actor)

        actor.sock().send(b"ss", b"ECHO", b"This is a string");

        result = c_char_p()
        actor.sock().recv(b"s", byref(result))

        self.assertEqual(string_at(result), b"This is a string")

        libc.free(result)

if __name__ == '__main__':
    unittest.main()



################################################################################
#  THIS FILE IS 100% GENERATED BY ZPROJECT; DO NOT EDIT EXCEPT EXPERIMENTALLY  #
#  Read the zproject/README.md for information about making permanent changes. #
################################################################################
from . import utils
from . import destructors
libczmq_destructors = destructors.lib

class Zdigest(object):
    """
    provides hashing functions (SHA-1 at present)
    """

    def __init__(self):
        """
        Constructor - creates new digest object, which you use to build up a
        digest by repeatedly calling zdigest_update() on chunks of data.
        """
        p = utils.lib.zdigest_new()
        if p == utils.ffi.NULL:
            raise MemoryError("Could not allocate person")

        # ffi.gc returns a copy of the cdata object which will have the
        # destructor called when the Python object is GC'd:
        # https://cffi.readthedocs.org/en/latest/using.html#ffi-interface
        self._p = utils.ffi.gc(p, libczmq_destructors.zdigest_destroy_py)

    def update(self, buffer, length):
        """
        Add buffer into digest calculation
        """
        utils.lib.zdigest_update(self._p, buffer, length)

    def data(self):
        """
        Return final digest hash data. If built without crypto support,
        returns NULL.
        """
        return utils.lib.zdigest_data(self._p)

    def size(self):
        """
        Return final digest hash size
        """
        return utils.lib.zdigest_size(self._p)

    def string(self):
        """
        Return digest as printable hex string; caller should not modify nor
        free this string. After calling this, you may not use zdigest_update()
        on the same digest. If built without crypto support, returns NULL.
        """
        return utils.lib.zdigest_string(self._p)

    @staticmethod
    def test(verbose):
        """
        Self test of this class.
        """
        utils.lib.zdigest_test(verbose)

################################################################################
#  THIS FILE IS 100% GENERATED BY ZPROJECT; DO NOT EDIT EXCEPT EXPERIMENTALLY  #
#  Read the zproject/README.md for information about making permanent changes. #
################################################################################



################################################################################
#  THIS FILE IS 100% GENERATED BY ZPROJECT; DO NOT EDIT EXCEPT EXPERIMENTALLY  #
#  Read the zproject/README.md for information about making permanent changes. #
################################################################################
from . import utils
from . import destructors
libczmq_destructors = destructors.lib

class ZdirPatch(object):
    """
    work with directory patches
    """

    def __init__(self, path, file, op, alias):
        """
        Create new patch
        """
        p = utils.lib.zdir_patch_new(utils.to_bytes(path), file._p, op, utils.to_bytes(alias))
        if p == utils.ffi.NULL:
            raise MemoryError("Could not allocate person")

        # ffi.gc returns a copy of the cdata object which will have the
        # destructor called when the Python object is GC'd:
        # https://cffi.readthedocs.org/en/latest/using.html#ffi-interface
        self._p = utils.ffi.gc(p, libczmq_destructors.zdir_patch_destroy_py)

    def dup(self):
        """
        Create copy of a patch. If the patch is null, or memory was exhausted,
        returns null.
        """
        return utils.lib.zdir_patch_dup(self._p)

    def path(self):
        """
        Return patch file directory path
        """
        return utils.lib.zdir_patch_path(self._p)

    def file(self):
        """
        Return patch file item
        """
        return utils.lib.zdir_patch_file(self._p)

    def op(self):
        """
        Return operation
        """
        return utils.lib.zdir_patch_op(self._p)

    def vpath(self):
        """
        Return patch virtual file path
        """
        return utils.lib.zdir_patch_vpath(self._p)

    def digest_set(self):
        """
        Calculate hash digest for file (create only)
        """
        utils.lib.zdir_patch_digest_set(self._p)

    def digest(self):
        """
        Return hash digest for patch file
        """
        return utils.lib.zdir_patch_digest(self._p)

    @staticmethod
    def test(verbose):
        """
        Self test of this class.
        """
        utils.lib.zdir_patch_test(verbose)

################################################################################
#  THIS FILE IS 100% GENERATED BY ZPROJECT; DO NOT EDIT EXCEPT EXPERIMENTALLY  #
#  Read the zproject/README.md for information about making permanent changes. #
################################################################################



################################################################################
#  THIS FILE IS 100% GENERATED BY ZPROJECT; DO NOT EDIT EXCEPT EXPERIMENTALLY  #
#  Read the zproject/README.md for information about making permanent changes. #
################################################################################
from . import utils
from . import destructors
libczmq_destructors = destructors.lib

class Zhash(object):
    """
    generic type-free hash container (simple)
    """

    def __init__(self):
        """
        Create a new, empty hash container
        """
        p = utils.lib.zhash_new()
        if p == utils.ffi.NULL:
            raise MemoryError("Could not allocate person")

        # ffi.gc returns a copy of the cdata object which will have the
        # destructor called when the Python object is GC'd:
        # https://cffi.readthedocs.org/en/latest/using.html#ffi-interface
        self._p = utils.ffi.gc(p, libczmq_destructors.zhash_destroy_py)

    @staticmethod
    def unpack(frame):
        """
        Unpack binary frame into a new hash table. Packed data must follow format
        defined by zhash_pack. Hash table is set to autofree. An empty frame
        unpacks to an empty hash table.
        """
        return utils.lib.zhash_unpack(frame._p)

    def insert(self, key, item):
        """
        Insert item into hash table with specified key and item.
        If key is already present returns -1 and leaves existing item unchanged
        Returns 0 on success.
        """
        return utils.lib.zhash_insert(self._p, utils.to_bytes(key), item._p)

    def update(self, key, item):
        """
        Update item into hash table with specified key and item.
        If key is already present, destroys old item and inserts new one.
        Use free_fn method to ensure deallocator is properly called on item.
        """
        utils.lib.zhash_update(self._p, utils.to_bytes(key), item._p)

    def delete(self, key):
        """
        Remove an item specified by key from the hash table. If there was no such
        item, this function does nothing.
        """
        utils.lib.zhash_delete(self._p, utils.to_bytes(key))

    def lookup(self, key):
        """
        Return the item at the specified key, or null
        """
        return utils.lib.zhash_lookup(self._p, utils.to_bytes(key))

    def rename(self, old_key, new_key):
        """
        Reindexes an item from an old key to a new key. If there was no such
        item, does nothing. Returns 0 if successful, else -1.
        """
        return utils.lib.zhash_rename(self._p, utils.to_bytes(old_key), utils.to_bytes(new_key))

    def freefn(self, key, free_fn):
        """
        Set a free function for the specified hash table item. When the item is
        destroyed, the free function, if any, is called on that item.
        Use this when hash items are dynamically allocated, to ensure that
        you don't have memory leaks. You can pass 'free' or NULL as a free_fn.
        Returns the item, or NULL if there is no such item.
        """
        return utils.lib.zhash_freefn(self._p, utils.to_bytes(key), free_fn)

    def size(self):
        """
        Return the number of keys/items in the hash table
        """
        return utils.lib.zhash_size(self._p)

    def dup(self):
        """
        Make copy of hash table; if supplied table is null, returns null.
        Does not copy items themselves. Rebuilds new table so may be slow on
        very large tables. NOTE: only works with item values that are strings
        since there's no other way to know how to duplicate the item value.
        """
        return utils.lib.zhash_dup(self._p)

    def keys(self):
        """
        Return keys for items in table
        """
        return utils.lib.zhash_keys(self._p)

    def first(self):
        """
        Simple iterator; returns first item in hash table, in no given order,
        or NULL if the table is empty. This method is simpler to use than the
        foreach() method, which is deprecated. To access the key for this item
        use zhash_cursor(). NOTE: do NOT modify the table while iterating.
        """
        return utils.lib.zhash_first(self._p)

    def next(self):
        """
        Simple iterator; returns next item in hash table, in no given order,
        or NULL if the last item was already returned. Use this together with
        zhash_first() to process all items in a hash table. If you need the
        items in sorted order, use zhash_keys() and then zlist_sort(). To
        access the key for this item use zhash_cursor(). NOTE: do NOT modify
        the table while iterating.
        """
        return utils.lib.zhash_next(self._p)

    def cursor(self):
        """
        After a successful first/next method, returns the key for the item that
        was returned. This is a constant string that you may not modify or
        deallocate, and which lasts as long as the item in the hash. After an
        unsuccessful first/next, returns NULL.
        """
        return utils.lib.zhash_cursor(self._p)

    def comment(self, format, *format_args):
        """
        Add a comment to hash table before saving to disk. You can add as many
        comment lines as you like. These comment lines are discarded when loading
        the file. If you use a null format, all comments are deleted.
        """
        utils.lib.zhash_comment(self._p, format, *format_args)

    def pack(self):
        """
        Serialize hash table to a binary frame that can be sent in a message.
        The packed format is compatible with the 'dictionary' type defined in
        http://rfc.zeromq.org/spec:35/FILEMQ, and implemented by zproto:

           ; A list of name/value pairs
           dictionary      = dict-count *( dict-name dict-value )
           dict-count      = number-4
           dict-value      = longstr
           dict-name       = string

           ; Strings are always length + text contents
           longstr         = number-4 *VCHAR
           string          = number-1 *VCHAR

           ; Numbers are unsigned integers in network byte order
           number-1        = 1OCTET
           number-4        = 4OCTET

        Comments are not included in the packed data. Item values MUST be
        strings.
        """
        return utils.lib.zhash_pack(self._p)

    def save(self, filename):
        """
        Save hash table to a text file in name=value format. Hash values must be
        printable strings; keys may not contain '=' character. Returns 0 if OK,
        else -1 if a file error occurred.
        """
        return utils.lib.zhash_save(self._p, utils.to_bytes(filename))

    def load(self, filename):
        """
        Load hash table from a text file in name=value format; hash table must
        already exist. Hash values must printable strings; keys may not contain
        '=' character. Returns 0 if OK, else -1 if a file was not readable.
        """
        return utils.lib.zhash_load(self._p, utils.to_bytes(filename))

    def refresh(self):
        """
        When a hash table was loaded from a file by zhash_load, this method will
        reload the file if it has been modified since, and is "stable", i.e. not
        still changing. Returns 0 if OK, -1 if there was an error reloading the
        file.
        """
        return utils.lib.zhash_refresh(self._p)

    def autofree(self):
        """
        Set hash for automatic value destruction. Note that this assumes that
        values are NULL-terminated strings. Do not use with different types.
        """
        utils.lib.zhash_autofree(self._p)

    @staticmethod
    def test(verbose):
        """
        Self test of this class.
        """
        utils.lib.zhash_test(verbose)

################################################################################
#  THIS FILE IS 100% GENERATED BY ZPROJECT; DO NOT EDIT EXCEPT EXPERIMENTALLY  #
#  Read the zproject/README.md for information about making permanent changes. #
################################################################################



################################################################################
#  THIS FILE IS 100% GENERATED BY ZPROJECT; DO NOT EDIT EXCEPT EXPERIMENTALLY  #
#  Read the zproject/README.md for information about making permanent changes. #
################################################################################
try:
    from . import native
    lib = native.lib
    ffi = native.ffi
except ImportError:
    from . import dlopen
    lib = dlopen.lib
    ffi = dlopen.ffi

def rebind (_lib, _ffi):
    """Rebind Lib and CompiledFFi objects for given package. Lib object MUST expose all functions
       required by the classes. This is default for zproject based projects"""
    global lib
    global ffi
    lib = _lib
    ffi = _ffi

try:
    text_type = unicode  # Python 2
    binary_type = str
except NameError:
    text_type = str      # Python 3
    binary_type = bytes


def to_bytes(s):
    return s if isinstance(s, binary_type) else text_type(s).encode("utf-8")


def to_unicode(s):
    return s if isinstance(s, text_type) else binary_type(s).decode("utf-8")


def to_strings (s):
    """Convert Python native list types to zlist_t of strings"""
    if issubclass (s.__class__, (list, set, frozenset, tuple)):
        foo = lib.zlist_new ()
        lib.zlist_autofree (foo)
        for item in s:
            lib.zlist_append (foo, to_bytes (item))
        return foo
    return None
################################################################################
#  THIS FILE IS 100% GENERATED BY ZPROJECT; DO NOT EDIT EXCEPT EXPERIMENTALLY  #
#  Read the zproject/README.md for information about making permanent changes. #
################################################################################



################################################################################
#  THIS FILE IS 100% GENERATED BY ZPROJECT; DO NOT EDIT EXCEPT EXPERIMENTALLY  #
#  Read the zproject/README.md for information about making permanent changes. #
################################################################################
from . import utils
from .Zactor import *
from .Zargs import *
MODE_BASE64_STD = 0 # standard base 64
MODE_BASE64_URL = 1 # url and filename friendly base 64
MODE_BASE32_STD = 2 # standard base 32
MODE_BASE32_HEX = 3 # extended hex base 32
MODE_BASE16 = 4 # standard base 16
MODE_Z85 = 5 # z85 from zeromq rfc 32
from .Zarmour import *
from .Zcert import *
from .Zcertstore import *
from .Zchunk import *
from .Zclock import *
from .Zconfig import *
from .Zdigest import *
from .Zdir import *
CREATE = 1 # creates a new file
DELETE = 2 # delete a file
from .ZdirPatch import *
from .Zfile import *
MORE = 1 #
REUSE = 2 #
DONTWAIT = 4 #
from .Zframe import *
from .Zhash import *
from .Zhashx import *
from .Ziflist import *
from .Zlist import *
from .Zlistx import *
from .Zloop import *
from .Zmsg import *
from .Zpoller import *
from .Zproc import *
from .Zsock import *
from .Zstr import *
from .Zsys import *
from .Ztimerset import *
from .Ztrie import *
from .Zuuid import *
from .ZhttpClient import *
from .ZhttpServer import *
from .ZhttpServerOptions import *
from .ZhttpRequest import *
from .ZhttpResponse import *
from .Zosc import *



################################################################################
#  THIS FILE IS 100% GENERATED BY ZPROJECT; DO NOT EDIT EXCEPT EXPERIMENTALLY  #
#  Read the zproject/README.md for information about making permanent changes. #
################################################################################
from . import utils
from . import destructors
libczmq_destructors = destructors.lib

class Ziflist(object):
    """
    List of network interfaces available on system
    """

    def __init__(self):
        """
        Get a list of network interfaces currently defined on the system
        """
        p = utils.lib.ziflist_new()
        if p == utils.ffi.NULL:
            raise MemoryError("Could not allocate person")

        # ffi.gc returns a copy of the cdata object which will have the
        # destructor called when the Python object is GC'd:
        # https://cffi.readthedocs.org/en/latest/using.html#ffi-interface
        self._p = utils.ffi.gc(p, libczmq_destructors.ziflist_destroy_py)

    def reload(self):
        """
        Reload network interfaces from system
        """
        utils.lib.ziflist_reload(self._p)

    def size(self):
        """
        Return the number of network interfaces on system
        """
        return utils.lib.ziflist_size(self._p)

    def first(self):
        """
        Get first network interface, return NULL if there are none
        """
        return utils.lib.ziflist_first(self._p)

    def next(self):
        """
        Get next network interface, return NULL if we hit the last one
        """
        return utils.lib.ziflist_next(self._p)

    def address(self):
        """
        Return the current interface IP address as a printable string
        """
        return utils.lib.ziflist_address(self._p)

    def broadcast(self):
        """
        Return the current interface broadcast address as a printable string
        """
        return utils.lib.ziflist_broadcast(self._p)

    def netmask(self):
        """
        Return the current interface network mask as a printable string
        """
        return utils.lib.ziflist_netmask(self._p)

    def print_py(self):
        """
        Return the list of interfaces.
        """
        utils.lib.ziflist_print(self._p)

    @staticmethod
    def new_ipv6():
        """
        Get a list of network interfaces currently defined on the system
        Includes IPv6 interfaces
        """
        return utils.lib.ziflist_new_ipv6()

    def reload_ipv6(self):
        """
        Reload network interfaces from system, including IPv6
        """
        utils.lib.ziflist_reload_ipv6(self._p)

    def is_ipv6(self):
        """
        Return true if the current interface uses IPv6
        """
        return utils.lib.ziflist_is_ipv6(self._p)

    @staticmethod
    def test(verbose):
        """
        Self test of this class.
        """
        utils.lib.ziflist_test(verbose)

################################################################################
#  THIS FILE IS 100% GENERATED BY ZPROJECT; DO NOT EDIT EXCEPT EXPERIMENTALLY  #
#  Read the zproject/README.md for information about making permanent changes. #
################################################################################
